% mtk -- Matlab widget toolkit.
%
%      [...] = mtk(command, ...)
%
% MTk is a high-level command line interface for creating graphical
% user interfaces using Handle Graphics objects.
%
% First argument COMMAND is a string.  Below is a list of all public
% commands together with their meanings.  Enter 'mtk help COMMAND' to
% display the detailed help text for a command.
%
% Utility commands:
%   help              Display on-line help.
%   wash              Message string beautifier.
%   extent            Determine width and height of a string.
%   focus             Set input focus on handle graphics object.
%   color             Convert color to RGB intensity values.
%   icon              Return color data for drawing an icon.
%   caller            Return caller's script file name.
%   load              Load Matlab object from text file.
%   read              Read MTk layout from a file.
%
% Dialog boxes:
%   message           Create a modal message dialog box.
%   warning           Create a modal warning message dialog box.
%   error             Create a modal error message dialog box.
%   question          Create a modal yes/no question dialog box.
%   getfile           Open a file selection dialog box (read mode).
%   putfile           Open a file selection dialog box (write mode).
%   filesel           Open a generic file selection dialog box.
%
% Widget layout:
%   render            Render widgets.
%   move              Move graphics object.
%
% Widget commands:
%   figure            Create a figure widget.
%   axes              Create an axes widget.
%   pushbutton        Create a push button widget.
%   togglebutton      Create a toggle button widget.
%   radiobutton       Create a radio button widget.
%   checkbox          Create a check box widget.
%   edit              Create an edit widget.
%   text              Create a text widget.
%   slider            Create a slider widget.
%   frame             Create a frame widget.
%   listbox           Create a list box widget.
%   popupmenu         Create a pop-up menu widget.
%   menu              Create a menu widget.
%   menuitem          Create menu items in widgets.
%   contextmenu       Create a context menu widget.
%
% Miscellaneous:
%   demo              Run MTk demo.
%   version           Get release information.
%   interface         Query interface information.
%
% Other help topics:
%   geometry          Widget geometry specification.
%
% MTk is for people who don't need a guide to write graphical user
% interfaces.

%% mtk.m --- the preceding comment is the documentation string.

% Copyright (C) 2001--2008 Ralph Schleicher

% Author: Ralph Schleicher <rs@ralph-schleicher.de>
% Time-stamp: <2008-07-29 10:01:44 CEST>

% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2,
% or (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
% Boston, MA 02111-1307, USA.

% As a special exception, Ralph Schleicher gives permission to link
% the code of this program with MATLAB from The Mathworks, Inc. (or
% with modified versions of MATLAB that use the same license as
% MATLAB), and distribute linked combinations including the two.
% You must obey the GNU General Public License in all respects for
% all of the code used other than with MATLAB.  If you modify this
% file, you may extend this exception to your version of the file,
% but you are not obligated to do so.  If you do not wish to do so,
% delete this exception statement from your version.

%% Commentary:

% Public commands (accessible via the 'mtk' gateway command) start
% with 'mtk_', internal commands start with 'mtki_'.  It should be
% possible to append this file to your own program and ship it as
% a single file.

%% To do:

% Add grid layout.

%% Code:

% Program entry point.
function varargout = mtk(varargin)

  % Check number of arguments.
  narg = numel(varargin);
  if narg < 1
    error(nargchk(1, inf, narg));
  end

  try
    % Replace command argument with function handle,
    % then call it with the original arguments.
    varargin{1} = ['mtk_' varargin{1}];
    if nargout == 0
      clear('ans');
      feval(varargin{:});
      if exist('ans', 'var')
        varargout = {ans};
      end
    else
      varargout = cell(1, nargout);
      [varargout{:}] = feval(varargin{:});
    end
  catch
    % Filter out original error message.
    err = mtki_wash(lasterr, '-error');
    % Delete 'mtk_' prefix so that function names look
    % like MTk commands.
    err = strrep(err, 'mtk_', '');
    % Signal error.
    error(err);
  end

% mtk/demo
%
%      mtk('demo')
%
% Run MTk demo.
function mtk_demo

  f = mtk('figure', 0, '200x100', ...
          'Name', 'MTk Demo');

  mtk('menuitem', f, ...
      {'&File', ...
       {'&Open...^o', @mtki_demo_open}, ...
       {'Close^w', @mtki_demo_close}, ...
       '--', ...
       {'&Save^s', @mtki_demo_save, 0}, ...
       {'Save as...', @mtki_demo_save, 1}, ...
       '--', ...
       {'Exit', @mtki_demo_exit}}, ...
      {'Figure', ...
       strvcat('&Save', 'filemenufcn(gcbf, ''FileSave'')'), ...
       strvcat('Save &as...', 'filemenufcn(gcbf, ''FileSaveAs'')'), ...
       strvcat('&Export...', 'filemenufcn(gcbf, ''FileExport'')'), ...
       '--', ...
       strvcat('Pa&ge setup...', 'pagesetupdlg(gcbf)'), ...
       strvcat('Print set&up...', 'printdlg(''-setup'', gcbf)'), ...
       strvcat('Print pre&view...', 'printpreview(gcbf)'), ...
       strvcat('&Print...', 'printdlg(gcbf)')}, ...
      {'&Help', ...
       {'About', @mtki_demo_about}, ...
       {'Demos', ...
        {'Widget alignment', @mtki_demo_alignment}, ...
        {'Input focus', @mtki_demo_focus}, ...
        {'Layout engine', @mtki_demo_layout}, ...
        {'Tab panel', @mtki_demo_tab_panel}, ...
        {'File selection', ...
         {'Open file', @mtki_demo_filesel, 'open'}, ...
         {'Save file', @mtki_demo_filesel, 'save'}, ...
         {'Select directory', @mtki_demo_filesel, 'dir'}}}});

  h1 = mtk('frame', f, '100x100+0+0');
  h2 = mtk('frame', f, '100x100-0+0');

  mtk('text', h1, [0 0], ...
      'String', 'Hello, world!');

  mtk('pushbutton', h2, '40x40+8+8', ...
      'String', 'one', 'Callback', 'mtk message');

  mtk('pushbutton', h2, '40x40-8+8', ...
      'String', 'two', 'Callback', 'mtk warning');

  mtk('pushbutton', h2, '40x40+8-8', ...
      'String', 'three', 'Callback', 'mtk error');

  mtk('pushbutton', h2, '40x40-8-8', ...
      'String', 'four', 'Callback', 'mtk question');

% MTk demo call-back procedures.
function mtki_demo_open(obj, event)

  file = mtk('getfile');
  if ~ file
    return;
  end
  set(gcbf, 'UserData', file);

function mtki_demo_close(obj, event)

  set(gcbf, 'UserData', []);

function mtki_demo_save(obj, event, save_as)

  file = get(gcbf, 'UserData');
  if save_as
    file = [];
  end
  if ~ file
    file = mtk('putfile');
    if ~ file
      return;
    end
  end
  set(gcbf, 'UserData', file);

function mtki_demo_exit(obj, event)

  delete(gcbf);

function mtki_demo_about(obj, event)

  [cur, rev, age] = mtk('interface');

  mtk('message', 'MTk Demo: About', ...
      sprintf(['%s\n' ...
               '\n', ...
               'Author: Ralph Schleicher\n', ...
               'Interface version: %d:%d:%d'], ...
              mtk('version', 'string'), cur, rev, age));

function mtki_demo_alignment(obj, event)

  f = mtk('figure', 0, '300x200', ...
          'Name', 'MTk Demo: Widget Alignment', ...
          'Color', [0.5 0.5 0.5]);

  h = mtk('text', f, '200x100', ...
          'BackgroundColor', 'w');

  % Internal widget alignment.
  mtk('text', h, '0x0+0+0', 'String', '+0+0');
  mtk('text', h, '0x0+0=0', 'String', '+0=0');
  mtk('text', h, '0x0+0-0', 'String', '+0-0');
  mtk('text', h, '0x0=0+0', 'String', '=0+0');
  mtk('text', h, '0x0=0=0', 'String', '=0=0');
  mtk('text', h, '0x0=0-0', 'String', '=0-0');
  mtk('text', h, '0x0-0+0', 'String', '-0+0');
  mtk('text', h, '0x0-0=0', 'String', '-0=0');
  mtk('text', h, '0x0-0-0', 'String', '-0-0');

  % External widget alignment.
  mtk('text', h, '0x0<0<0', 'String', '<0<0');
  mtk('text', h, '0x0<0=0', 'String', '<0=0');
  mtk('text', h, '0x0<0>0', 'String', '<0>0');
  mtk('text', h, '0x0=0<0', 'String', '=0<0');
  mtk('text', h, '0x0=0>0', 'String', '=0>0');
  mtk('text', h, '0x0>0<0', 'String', '>0<0');
  mtk('text', h, '0x0>0=0', 'String', '>0=0');
  mtk('text', h, '0x0>0>0', 'String', '>0>0');

function mtki_demo_focus(obj, event)

  f = mtk('figure', 0, '300x200', ...
          'Name', 'MTk Demo: Input Focus');

  h = mtk('edit', f, '268x130+16+16', ...
          'Min', 1, 'Max', 23);

  mtk('pushbutton', h, '0x0=0>16', ...
      'Callback', {@mtki_demo_focus1, h}, ...
      'String', 'Set focus on edit control');

function mtki_demo_focus1(obj, event, h)

  mtk('focus', h);

function mtki_demo_layout(obj, event)

  f = mtk('figure', 0, '0x0', ...
          'Name', 'MTk Demo: Layout Engine', ...
          'CloseRequestFcn', {@mtki_demo_layout_close, 1}, ...
          'Resize', 'on', ...
          'Visible', 'off');

  mtk('render', f, ...
      'Margin', 16, 'Padding', 16, ...
      {'box', ...
       'Flexibility', 1.0, ...
       'Alignment', 'top', ...
       'Padding', 16, ...
       {'group', ...
        'Caption', 'Fruits:', ...
        {'box', ...
         'Margin', 8, 'Padding', 4, ...
         {'radiobutton', ...
          'String', 'Apples', ...
          'Callback', {@mtki_demo_layout_dirty, 1}}, ...
         {'radiobutton', ...
          'String', 'Oranges', ...
          'Callback', {@mtki_demo_layout_dirty, 1}}, ...
         {'radiobutton', ...
          'String', 'Bananas', ...
          'Callback', {@mtki_demo_layout_dirty, 1}}}}, ...
       {'group', ...
        'Caption', 'Input:', ...
        {'box', ...
         'Flexibility', 1.0, ...
         'Margin', 8, ...
         {'text', ...
          'String', 'File name:'}, ...
         {'box', ...
          'Padding', 8, ...
          {'edit', 'Tag', 'file_name', ...
           'Flexibility', 1.0, 'Width', 120, ...
           'Callback', {@mtki_demo_layout_dirty, 1}}, ...
          {'pushbutton', ...
           'String', 'Browse', ...
           'Callback', @mtki_demo_layout_browse}}}}}, ...
      {'box', ...
       'Pack', 'justify', ...
       'Padding', 16, ...
       {'pushbutton', ...
        'String', 'Close', ...
        'Callback', {@mtki_demo_layout_close, 1}}, ...
       {'pushbutton', ...
        'String', 'Apply', ...
        'Callback', {@mtki_demo_layout_dirty, 0}}, ...
       {'pushbutton', ...
        'String', 'Cancel', ...
        'Callback', {@mtki_demo_layout_close, 0}}, ...
       {'pushbutton', ...
        'String', 'Help', ...
        'Callback', 'mtk(''message'', ''No help available.'');'}});

  data = struct('dirty', 0, ...
                'handle', guihandles(f));

  set(f, 'UserData', data);

  mtk('focus', data.handle.file_name);

function mtki_demo_layout_close(obj, event, check)

  if check
    data = get(gcbf, 'UserData');
    if data.dirty
      mtk('question', 'Apply unsaved changes?');
    end
  end
  delete(gcbf);

function mtki_demo_layout_dirty(obj, event, dirty)

  data = get(gcbf, 'UserData');
  data.dirty = dirty;
  set(gcbf, 'UserData', data);

function mtki_demo_layout_browse(obj, event)

  file = mtk('getfile');
  if file
    data = get(gcbf, 'UserData');
    set(data.handle.file_name, 'String', file);
    data.dirty = 1;
    set(gcbf, 'UserData', data);
  end

function mtki_demo_tab_panel(obj, event)

  f = mtk('figure', 0, '0x0', ...
          'Name', 'MTk Demo: Tab Panel', ...
          'Resize', 'off', ...
          'Visible', 'off');

  mtk('render', f, ...
      {'vbox', ...
       'Margin', 16, ...
       'Padding', 16, ...
       {'tabs', ...
        {'tab', ...
         'Caption', 'First', ...
         {'vbox', ...
          'Margin', 8, ...
          {'text', ...
           'String', 'Hello world!'}}}, ...
        {'tab', ...
         'Caption', 'Second', ...
         {'vbox', ...
          'Margin', 8, ...
          {'text', ...
           'String', 'Good by sailor!'}}}, ...
        {'tab', ...
         'Caption', 'Third', ...
         {'box', ...
          'Margin', 8, ...
          {'tabs', ...
           {'tab', ...
            'Caption', 'Fourth', ...
            {'vbox', ...
             'Margin', 8, ...
             {'group', ...
              'Caption', 'Fruits:', ...
              {'vbox', ...
               'Margin', 8, ...
               {'checkbox', ...
                'String', 'Apples'}, ...
               {'checkbox', ...
                'String', 'Bananas'}}}}}, ...
           {'tab', ...
            'Caption', 'Fifth', ...
            {'vbox', ...
             'Margin', 8, ...
             {'text', ...
              'String', 'The end.'}}}}}}}, ...
       {'hbox', ...
        'Pack', 'center', ...
        {'pushbutton', ...
         'String', 'Close', ...
         'Callback', 'delete(gcbf);'}}});

function mtki_demo_filesel(obj, event, mode)

  switch mode
   case 'open'
    mtk('filesel', ...
        'Style', 'open', ...
        'SelectionType', 'multiple', ...
        'Filter', {'*.m', 'Matlab M-files'; ...
                   '*.mdl', 'Simulink models'})
   case 'save'
    mtk('filesel', ...
        'Style', 'save', ...
        'DefaultFilename', 'default.m')
   case 'dir'
    mtk('filesel', ...
        'Style', 'dir')
  end

% mtk/version
%
%     str = mtk('version', what)
%
% Get version information.
%
% Optional argument WHAT specifies what to return.  Value is either 'name',
%  'version', 'string', or 'address'.  The default is 'version'.
function str = mtk_version(what)

  if nargin == 0
    what = 'version';
  end

  switch what
   case 'name'
    str = 'MTk';
   case 'version'
    str = '2008-07-29';
   case 'string'
    str = 'MTk 2008-07-29';
   case 'address'
    str = 'rs@ralph-schleicher.de';
   otherwise
    error('Invalid argument');
  end

% mtk/interface
%
%      [current, revision, age] = mtk('interface')
%      p = mtk('interface', interface, revision)
%
% Query interface information.
%
% When called without any arguments, the interface command returns up
% to three values:
%
% First return value CURRENT is the most recent interface number
%  that this release of MTk implements.
% Second return value REVISION is the implementation number of
%  the CURRENT interface.
% Third return value AGE is the difference between the newest and
%  oldest interfaces that this release of MTk implements.  In other
%  words, this release of MTk implements all the interface numbers
%  in the range from number 'CURRENT - AGE' to 'CURRENT'.
%
% When called with arguments, the interface command returns non-zero
% if this release of MTk is compatible with the required interface:
%
% First argument INTERFACE is the required interface number.
% Optional second argument REVISION is the minimum revision number
%  for that interface.
%
% Please note that the interface command is not available in the first
% MTk release.  Use 'eval' to check for this feature.
%
% Example:
%
%      if ~ eval('mtk(''interface'', 1)', '0')
%        error('Require MTk interface version 1');
%      end
function [current, revision, age] = mtk_interface(interface, rev)

  % How to update the MTk interface information:
  %
  %   1. Start with interface (0, 0, 0).
  %   2. If the source code has changed at all since the last update,
  %      then increment REVISION.
  %   3. If any interfaces have been added, removed, or changed since
  %      the last update, increment CURRENT, and set REVISION to 0.
  %   4. If any interfaces have been added since the last public release,
  %      then increment AGE.
  %   5. If any interfaces have been removed or changed since the last
  %      public release, then set AGE to 0.
  [current, revision, age] = deal(3, 0, 1);

  switch nargin
   case 0
    return;
   case 1
    rev = 0;
  end

  if nargout > 1
    error('Too many output arguments');
  end

  if numel(interface) ~= 1 | rem(interface, 1) ~= 0
    error('Invalid argument');
  end
  if interface > current
    p = 0;
  elseif interface == current
    if numel(rev) ~= 1 | rem(rev, 1) ~= 0
      error('Invalid argument');
    end
    p = (rev <= revision);
  else
    p = (interface >= current - age);
  end

  current = p;

% mtk/help
%
%      mtk('help', topic)
%      mtk('help')
%
% Display the help text for a MTk command.
%
% If the first argument TOPIC is omitted, display the help text for
% the gateway command.
function mtk_help(topic)

  % Display help text.
  if nargin == 0
    mtki_help(mfilename);
  else
    mtki_help(mfilename, topic);
  end

% Show primary or secondary help texts.
%
% First argument PRI is the name of a Matlab command documented in
% an M-file (the primary help topic).  Optional second argument SEC
% specifies a secondary help topic for that command.
%
% Return value is the help text (a string).  If the return value is
% omitted, the help text is displayed in the command window.
%
% Secondary help texts are encoded in the M-file like the primary
% help text, except that the beginning of the help text is marked
% by the character sequence '% PRI/SEC' on a line by itself.
function varargout = mtki_help(pri, sec)

  % Assign optional arguments.
  if nargin < 2
    sec = [];
  end

  % Locate M-file.
  file = which([pri '.m']);
  if isempty(file)
    error(sprintf('No such file ''%s.m''', pri));
  end

  % Control characters.
  nl = sprintf('\n');

  % Primary help text.
  s = mtki_wash(help(pri), '-eol', '-final');
  if isempty(s)
    % No help available.
  elseif isempty(sec)
    % Show primary help text.
  else
    % Extract H1 line.
    ind = strfind(s, nl);
    h1 = s(1:ind(1));
    % Search beginning of help text.
    s = mtki_wash(fileread(file), '-eol', '-final');
    ind = strfind(s, sprintf('%% %s/%s\n', pri, sec));
    if isempty(ind)
      s = [];
    else
      s = s(ind(1):end);
      % Find end of help text.
      ind = strfind(s, nl);
      n = numel(ind);
      for i = 1:n-1
        if s(ind(i)+1) ~= double('%')
          s = s(1:ind(i));
          ind = ind(1:i);
          break;
        end
      end
      % Extract header line and body.
      % Replace comment characters.
      head = [' ' s(2:ind(1)-1)];
      s(ind+1) = double(' ');
      body = s(ind(1)+1:end);
      % Merge header and H1 line.
      ind = strfind(lower(h1), lower(pri));
      if ~ isempty(ind)
        h1 = [head h1(ind(1)+numel(pri):end)];
      end
      % Join H1 line and body.
      s = [h1 body];
    end
  end

  if nargout == 0
    % Show help.
    if isempty(s)
      fprintf('No help available.\n');
    else
      fprintf('\n%s\n', s);
    end
  else
    % Remove superfluous indentation and whitespace.
    if ~ isempty(s)
      % Indentation of first line.
      n = 1;
      while s(n) == double(' ')
        n = n + 1;
      end
      n = n - 1;
      if n > 0
        % Delete not more than N spaces at beginning
        % of each line (caveat, walk backwards).
        ind = strfind(s, nl);
        for i = ind(end-1:-1:1)
          j = 1;
          while (s(i+j) == double(' ')) & (j <= n)
            j = j + 1;
          end
          j = j - 1;
          s(i+1:i+j) = [];
        end
      end
      s = mtki_wash(s, '-trim');
    end
    % Return value.
    varargout = {s};
  end

% Split file name into directory name and base file name.
%
% Argument FILENAME is a file name.
function [dirname, basename] = mtki_fileparts(filename)

  % Directory name.
  [dirname, name, ext] = fileparts(filename);

  % Base file name.
  basename = [name, ext];

% Return canonical directory name.
%
% Argument DIRNAME is a directory.  If DIRNAME is empty, return the
%  current working directory.
%
% Return value is empty if an error occurs.
function pathname = mtki_dirpath(dirname)

  if isempty(dirname)
    pathname = cd;
  else
    try
      here = cd;
      cd(dirname);
      pathname = cd;
      cd(here);
    catch
      pathname = [];
    end
  end

% mtk/wash
%
%      w = mtk('wash', s, ...)
%
% Perform various text conversion routines.
%
% First argument S is the input string.  The rest of the argument list
% are options specifying washing functions to be applied to S.  Default
% is to trim the input string.  Below is a list of all known washing
% machines, together with their meaning.
%
% -trim
%      Remove leading and trailing whitespace characters.
% -eol
%      Perform end-of-line conversion from DOS and Mac style end-of-line
%      sequences to new-line characters.
% -chop
%      Remove any trailing end-of-line sequences.
% -error
%      Delete all lines containing the character sequence ' ==> '.
%      This is useful for washing Matlab error messages.
% -period
%      Add a period unless the string ends with a period, exclamation mark,
%      or question mark.
% -final
%      Ensure that the string ends in a new-line character.
%
% Return value is the washed string.
function s = mtk_wash(in, varargin)

  % Check number of arguments.
  error(nargchk(1, inf, nargin));

  if numel(varargin) == 0
    s = mtki_wash(in, '-trim');
  else
    s = mtki_wash(in, varargin{:});
  end

% Text conversion.
function s = mtki_wash(in, varargin)

  % Non-zero means to remove leading
  % and trailing whitespace characters.
  trim = 0;

  % Non-zero means to perform
  % end-of-line conversion.
  eol = 0;

  % Non-zero means to remove any trailing
  % end-of-line sequences.
  chop = 0;

  % Non-zero means to remove error lines.
  err = 0;

  % Non-zero means to add a period.
  period = 0;

  % Non-zero means to ensure that the string
  % ends in a new-line.
  final = 0;

  % Get options.
  for i = 1:numel(varargin)
    switch varargin{i}
     case '-trim'
      trim = 1;
     case '-eol'
      eol = 1;
     case '-chop'
      chop = 1;
     case '-error'
      trim = 1;
      err = 1;
     case '-period'
      trim = 1;
      period = 1;
     case '-final'
      final = 1;
     otherwise
      error('Unknown option');
    end
  end

  % Control characters.
  cr = sprintf('\r');
  nl = sprintf('\n');

  % Input string.
  s = in;

  % Remove leading and trailing whitespace characters.
  if trim
    ind = find(~ isspace(s));
    if ind
      s = s(min(ind):max(ind));
    end
  end

  % Perform end-of-line conversion.
  if eol
    % DOS end-of-line conversion.
    ind = strfind(s, [cr nl]);
    if ind
      s(ind) = [];
    end
    % Mac end-of-line conversion.
    ind = strfind(s, cr);
    if ind
      s(ind) = nl;
    end
  end

  % Remove any trailing end-of-line sequences.
  if chop
    n = numel(s);
    while n > 0
      if (s(n) == cr) | (s(n) == nl)
        n = n - 1;
      else
        break;
      end
    end
    s(n+1:end) = [];
  end

  % Delete lines containing ' ==> '.
  if err
    ind = strfind(s, nl);
    if isempty(ind)
      bol = 1;
      eol = numel(s);
    else
      bol = [1 ind];
      eol = [ind-1 numel(s)];
    end
    pos = strfind(s, ' ==> ');
    for j = pos(end:-1:1)
      ind = find(j > bol & j < eol);
      s(bol(ind):eol(ind)) = [];
    end
  end

  % Check if the message ends correctly.
  if period
    if numel(s)
      switch s(end)
       case {'.', '!', '?'}
       otherwise
        s = [s '.'];
      end
    end
  end

  % Ensure that the string ends in a new-line.
  if final
    n = numel(s);
    if n > 0
      if s(n) ~= nl
        s = [s nl];
      end
    else
      s = nl;
    end
  end

% mtk/extent
%
%      [width, height] = mtk('extent', s)
%
% Determine the actual width and height of the string S.
%
% Argument S can be a string, character array, or cell array of strings.
% Line breaks occur after any '\n' character sequence embedded in a string,
% each row of a character array, and each cell of a cell array of strings.
function [width, height] = mtk_extent(s)

  % Check number of arguments.
  error(nargchk(1, 1, nargin));

  [width, height] = mtki_extent(s);

% Determine width and height of a text string.
function [w, h] = mtki_extent(s)

  persistent extra width height

  % Initialize lookup tables.
  if isempty(extra)
    f = figure('Visible', 'off');
    t = uicontrol('Parent', f, ...
                  'Style', 'text', ...
                  'Units', 'pixels');
    % Extra space.
    set(t, 'String', []);
    extent = get(t, 'Extent');
    extra = extent(3:4);
    % Character width and height.
    width = zeros(1, 256);
    height = zeros(1, 256);
    for i = 1:256
      set(t, 'String', char(i - 1));
      extent = get(t, 'Extent');
      width(i) = extent(3) - extra(1);
      height(i) = extent(4) - extra(2);
    end
    delete(f);
  end

  % Convert multi-line string into cell array of strings.
  if ischar(s)
    if size(s, 1) > 1
      s = cellstr(s);
    elseif strfind(s, '\n');
      s = strrep(s, '''', '''''');
      s = strrep(s, '\n', ''', ''');
      s = eval(['{''' s '''}']);
    end
  end

  if isempty(s)
    [w, h] = deal(0);
  elseif iscell(s)
    [w, h] = deal(0);
    for i = 1:numel(s)
      ind = s{i} + 1;
      w = max(w, sum(width(ind)));
      h = h + max(height(ind));
    end
  else
    ind = s + 1;
    w = sum(width(ind));
    h = max(height(ind));
  end

  w = w + extra(1);
  h = h + extra(2);

% mtk/focus
%
%      mtk('focus', h)
%
% Set the input focus on handle graphics object H.
function mtk_focus(h)

  if nargin < 1
    error(nargchk(1, 1, nargin));
  end

  if numel(h) ~= 1
    error('Invalid argument');
  end

  % Get figure handle.
  fig = get(h, 'Parent');
  if isempty(fig)
    return;
  elseif fig == 0
    fig = h;
  end

  % Raise figure.
  figure(fig);

  % Determine the focus point.
  units = get(fig, 'Units');
  set(fig, 'Units', 'pixels');
  fig_pos = get(fig, 'Position');
  set(fig, 'Units', units);
  % Lower left corner of the figure window.
  point = fig_pos(1:2);
  if fig ~= h
    units = get(h, 'Units');
    set(h, 'Units', 'pixels');
    obj_pos = get(h, 'Position');
    set(h, 'Units', units);
    % Upper left corner of the Handle Graphics object.
    point = point + obj_pos(1:2) + [1 obj_pos(4)] - 2;
    if ispc
      % Get off by 2.
      point = point + [2 -2];
    end
  end

  % Save call-back handler.
  try
    callback = get(h, 'Callback');
    if ~ isempty(callback)
      set(h, 'Callback', '');
    end
  catch
    callback = [];
  end

  % Save pointer location.
  mark = get(0, 'PointerLocation');

  % Click on the focus point.
  units = get(0, 'Units');
  set(0, 'Units', 'pixels');
  set(0, 'PointerLocation', point);
  set(0, 'Units', units);

  try
    drawnow;
    mouseclick;
  end

  % Restore pointer location.
  set(0, 'PointerLocation', mark);

  % Restore call-back handler.
  if ~ isempty(callback)
    set(h, 'Callback', callback);
  end

% mtk/color
%
%      c = mtk('color', spec)
%
% Convert color specifications to RGB intensity values.
%
% Argument SPEC is either a single color specification or a cell
%  array of color specifications.  Each color specification is
%  either a standard Matlab color specification or a numerical
%  color value of an unsigned integral data type.
%
% Return value is a vector with three RGB intensity values.
% If SPEC is a cell array, the return value is a color map.
function c = mtk_color(spec)

  % Check number of arguments.
  if nargin < 1
    error(nargchk(1, 1, nargin));
  end

  if iscell(spec)
    n = numel(spec);
    c = zeros(n, 3);
    for i = 1:n
      c(i, :) = mtki_color(spec{i});
    end
  else
    c = mtki_color(spec);
  end

% Convert a color specification.
function color = mtki_color(color)

  if ischar(color)
    switch color
     case {'y', 'yellow'}
      color = [1, 1, 0];
     case {'m', 'magenta'}
      color = [1, 0, 1];
     case {'c', 'cyan'}
      color = [0, 1, 1];
     case {'r', 'red'}
      color = [1, 0, 0];
     case {'g', 'green'}
      color = [0, 1, 0];
     case {'b', 'blue'}
      color = [0, 0, 1];
     case {'w', 'white'}
      color = [1, 1, 1];
     case {'k', 'black'};
      color = [0, 0, 0];
     otherwise
      error('Invalid Matlab color name');
    end
  elseif any(numel(color) == [1, 3])
    switch class(color)
     case {'double', 'single'}
      if imag(color) ~= 0 | any(color < 0 | color > 1)
        error('Invalid numeric color value');
      end
      color = double(color);
     case 'logical'
      color = double(color);
     case 'uint8'
      color = double(color) ./ 255;
     case 'uint16'
      color = double(color) ./ 65535;
     case 'uint32'
      color = double(color) ./ 4294967295;
     otherwise
      error('Invalid color value data type');
    end
    if numel(color) == 1
      color = repmat(color, 1, 3);
    else
      color = reshape(color, 1, 3);
    end
  else
    error('Invalid Matlab color specification');
  end

% mtk/icon
%
%      im = mtk('icon', icon, background)
%
% Return color data for drawing an icon.
%
% First argument ICON is the icon name.  Known built-in icon names are
%  'up', 'home', and 'dir'.  Otherwise, the image is read from a file
%  of that name.
% If the optional second argument BACKGROUND is given, the transparency
%  information of the image is merged with the color value defined by
%  BACKGROUND.  Argument BACKGROUND is either a color specification or
%  a widget handle.
function im = mtk_icon(icon, background)

  % Check number of arguments.
  if nargin < 1
    error(nargchk(1, 2, nargin));
  end
  if nargin < 2
    background = [];
  end

  % Get image data.
  [im, map, alpha] = deal([], [], []);

  switch icon
   case 'up'
    [im, alpha] = mtki_icon_up;
   case 'home'
    [im, alpha] = mtki_icon_home;
   case 'dir'
    [im, alpha] = mtki_icon_dir;
   otherwise
    try
      [im, map, alpha] = imread(icon);
    catch
      [im, map] = imread(icon);
    end
  end

  m = size(im, 1);
  n = size(im, 2);

  % Convert image data into color data.
  switch class(im)
   case 'logical'
    f = 1;
   case 'int8'
    f = 127;
   case 'uint8'
    f = 255;
   case 'int16'
    f = 32767;
   case 'uint16'
    f = 65535;
   case 'int32'
    f = 2147483647;
   case 'uint32'
    f = 4294967295;
   case {'double', 'single'}
    f = 1;
   otherwise
    error('Invalid image data type');
  end

  if isempty(map)
    % Direct-color.
    im = double(im);
    if f ~= 1
      im = im ./ f;
      % Fix rounding errors.
      im(im < 0) = 0;
      im(im > 1) = 1;
    end
    if ndims(im) == 2
      im = cat(3, im, im, im);
    end
  else
    % Pseudo-color.
    im = im(:);
    if any(im == 0)
      im = im + 1;
    end
    im = reshape([map(im, 1); map(im, 2); map(im, 3)], m, n, 3);
  end

  % Apply background transparency.
  if isempty(background) | isempty(alpha)
    return;
  end

  % Get background color.
  if numel(background) == 1 & ishandle(background)
    c = mtki_background(background);
  else
    c = background;
  end
  c = mtki_color(c);

  % Merge background color.
  alpha = double(alpha);
  if f ~= 1
    alpha = alpha ./ f;
    % Fix rounding errors.
    alpha(alpha < 0) = 0;
    alpha(alpha > 1) = 1;
  end
  im(:, :, 1) = im(:, :, 1) .* alpha + c(1) .* (1 - alpha);
  im(:, :, 2) = im(:, :, 2) .* alpha + c(2) .* (1 - alpha);
  im(:, :, 3) = im(:, :, 3) .* alpha + c(3) .* (1 - alpha);

% Go up.
function [im, alpha] = mtki_icon_up

  im = cat(3, ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan  28 nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan   0 197   0 nan nan nan nan nan nan; ...
     nan nan nan nan nan nan   0 196 159  42   0 nan nan nan nan nan; ...
     nan nan nan nan nan   0 196 156 160  63  42   0 nan nan nan nan; ...
     nan nan nan nan   0 196 166 164 158  63  63  42   0 nan nan nan; ...
     nan nan nan   0 196 163 170 163 158  63  63  63  42   0 nan nan; ...
     nan nan  43   0   0   0 171 159 156  63  41   0   0   0   0 nan; ...
     nan nan nan nan nan   0 175 155 159  63  41   0 nan nan nan nan; ...
     nan nan nan nan nan   0 171 155 159  63  41   0 nan nan nan nan; ...
     nan nan nan nan nan   0 174 159 159  63  41   0 nan nan nan nan; ...
     nan nan nan nan nan   0 163 154 159  63  41   0 nan nan nan nan; ...
     nan nan nan nan nan   0 104  73  75  52  45   0 nan nan nan nan; ...
     nan nan nan nan nan   0   0   0   0   0   0   0 nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan  45 nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan   0 213   0 nan nan nan nan nan nan; ...
     nan nan nan nan nan nan   0 212 185  67   0 nan nan nan nan nan; ...
     nan nan nan nan nan   0 212 183 186 101  67   0 nan nan nan nan; ...
     nan nan nan nan   0 212 190 189 184 101 101  67   0 nan nan nan; ...
     nan nan nan   0 212 188 193 188 184 101 101 101  67   0 nan nan; ...
     nan nan  43   0   0   0 194 185 183 101  66   0   0   0   0 nan; ...
     nan nan nan nan nan   0 197 182 185 101  66   0 nan nan nan nan; ...
     nan nan nan nan nan   0 194 182 185 101  66   0 nan nan nan nan; ...
     nan nan nan nan nan   0 196 185 185 101  66   0 nan nan nan nan; ...
     nan nan nan nan nan   0 188 181 185 101  66   0 nan nan nan nan; ...
     nan nan nan nan nan   0 146 117 120  83  71   0 nan nan nan nan; ...
     nan nan nan nan nan   0   0   0   0   0   0   0 nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan  61 nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan   0 228   0 nan nan nan nan nan nan; ...
     nan nan nan nan nan nan   0 227 210  91   0 nan nan nan nan nan; ...
     nan nan nan nan nan   0 227 209 211 136  91   0 nan nan nan nan; ...
     nan nan nan nan   0 227 213 213 209 136 136  91   0 nan nan nan; ...
     nan nan nan   0 227 212 215 212 209 136 136 136  91   0 nan nan; ...
     nan nan  43   0   0   0 216 210 209 136  90   0   0   0   0 nan; ...
     nan nan nan nan nan   0 218 208 210 136  90   0 nan nan nan nan; ...
     nan nan nan nan nan   0 216 208 210 136  90   0 nan nan nan nan; ...
     nan nan nan nan nan   0 217 210 210 136  90   0 nan nan nan nan; ...
     nan nan nan nan nan   0 212 207 210 136  90   0 nan nan nan nan; ...
     nan nan nan nan nan   0 185 158 162 111  96   0 nan nan nan nan; ...
     nan nan nan nan nan   0   0   0   0   0   0   0 nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan 255 nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan 255 255 255 nan nan nan nan nan nan; ...
     nan nan nan nan nan nan 255 255 255 255 255 nan nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 255 nan nan; ...
     nan nan 255 255 255 255 255 255 255 255 255 255 255 255 255 nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% Go home.
function [im, alpha] = mtki_icon_home

  im = cat(3, ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan  30   0  23 nan   0   0 nan nan nan nan nan nan nan; ...
     nan nan nan   0 169   0   0 101  91   0 nan nan nan nan nan nan; ...
     nan nan nan   0 118   0  96 151 233 102   0 nan nan nan nan nan; ...
     nan nan nan   0  17  57 139 201 233 196 155   0 nan nan nan nan; ...
     nan nan nan   0  74 155 233 196 233 201 233 133   0 nan nan nan; ...
     nan nan   0  74 180 201 233 201 233 201 233 206 155   0 nan nan; ...
     nan   0  25   0 234 202 234 207 232  46  63  52   0  20   0 nan; ...
     nan nan nan   0 223  80  87  61 239  57 255  50   0 nan nan nan; ...
     nan nan nan   0 224  83 165  83 237  46  57  46   0 nan nan nan; ...
     nan nan nan   0 223  83 165  83 228 188 220 187   0 nan nan nan; ...
     nan nan nan   0 226  83 165  83 219 181 210 182   0 nan nan nan; ...
     nan nan nan   0 219  97  83  67 204 187 211 182   0 nan nan nan; ...
     nan 181  66  66  88  91  70  54  39  11  86  73  53  51  73 nan; ...
     nan  66 120 120  73  73  73  73  73 151  73  73  73  73 nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan  30   0  23 nan   0   0 nan nan nan nan nan nan nan; ...
     nan nan nan   0  88   0   0 101  91   0 nan nan nan nan nan nan; ...
     nan nan nan   0  62   0  96 151 233 102   0 nan nan nan nan nan; ...
     nan nan nan   0  17  57 139 201 233 196 155   0 nan nan nan nan; ...
     nan nan nan   0  74 155 233 196 233 201 233 133   0 nan nan nan; ...
     nan nan   0  74 180 201 233 201 233 201 233 206 155   0 nan nan; ...
     nan   0  25   0 234 202 234 207 232  46  63  52   0  20   0 nan; ...
     nan nan nan   0 223  35  38  27 239  57 255  50   0 nan nan nan; ...
     nan nan nan   0 224  37  73  37 237  46  57  46   0 nan nan nan; ...
     nan nan nan   0 223  37  73  37 228 188 220 187   0 nan nan nan; ...
     nan nan nan   0 226  37  73  37 219 181 210 182   0 nan nan nan; ...
     nan nan nan   0 219  43  37  29 204 187 211 182   0 nan nan nan; ...
     nan 181  74  74 100 102  85  69  49  14  98  85  64  61  85 nan; ...
     nan  74 120 120  85  85  85  85  85 151  85  85  85  85 nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan  30   0  23 nan   0   0 nan nan nan nan nan nan nan; ...
     nan nan nan   0  73   0   0 101  91   0 nan nan nan nan nan nan; ...
     nan nan nan   0  47   0  96 151 233 102   0 nan nan nan nan nan; ...
     nan nan nan   0  17  57 139 201 233 196 155   0 nan nan nan nan; ...
     nan nan nan   0  74 155 233 196 233 201 233 133   0 nan nan nan; ...
     nan nan   0  74 180 201 233 201 233 201 233 206 155   0 nan nan; ...
     nan   0  25   0 234 202 234 207 232  46  63  52   0  20   0 nan; ...
     nan nan nan   0 223  26  29  20 239  57 255  50   0 nan nan nan; ...
     nan nan nan   0 224  27  55  27 237  46  57  46   0 nan nan nan; ...
     nan nan nan   0 223  27  55  27 228 188 220 187   0 nan nan nan; ...
     nan nan nan   0 226  27  55  27 219 181 210 182   0 nan nan nan; ...
     nan nan nan   0 219  32  27  21 204 187 211 182   0 nan nan nan; ...
     nan 181  64  64  85  87  65  48  34   9  82  69  49  47  69 nan; ...
     nan  64 120 120  69  69  69  69  69 151  69  69  69  69 nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan 255 255 255 nan 255 255 nan nan nan nan nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 nan nan nan nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 nan nan nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 nan nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan 255 255 255 255 255 255 255 255 255 255 255 255 nan nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan nan nan 255 255 255 255 255 255 255 255 255 255 nan nan nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% Directory.
function [im, alpha] = mtki_icon_dir

  im = cat(3, ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 142 142 142 142 102   0 nan nan nan nan nan nan nan nan; ...
     nan   0 161 132 132 131 130   0   0   0   0   0   0   0   0 163; ...
     nan   0 142 116 115 115 114 113 111 114 114 114 114 114 102   0; ...
     nan   0 142 104 104 103 103  86   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0   0   0 162 216 214 210 209 208 186   0; ...
     nan   0 214 214 214 214 214 214 219 195 193 189 187 185 165   0; ...
     nan   0 208 200 200 200 200 188 188 188 191 188 186 184 164   0; ...
     nan   0 215 201 200 197 195 193 192 189 188 185 184 181 161   0; ...
     nan   0 215 198 197 194 193 191 189 187 185 182 181 179 158   0; ...
     nan   0 215 193 191 188 187 185 184 181 179 177 175 172 153   0; ...
     nan   0 215 188 186 183 183 180 178 175 173 171 169 165 147   0; ...
     nan   0 215 182 180 178 176 174 172 168 167 165 163 159 142   0; ...
     nan   0 193 161 159 156 155 153 151 148 147 145 143 140 139   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 146 146 146 146 105   0 nan nan nan nan nan nan nan nan; ...
     nan   0 164 135 134 133 132   0   0   0   0   0   0   0   0 163; ...
     nan   0 146 119 119 118 117 115 114 114 117 117 117 117 105   0; ...
     nan   0 146 106 106 105 105  88   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0   0   0 163 217 215 212 211 209 187   0; ...
     nan   0 215 215 215 215 215 215 220 197 195 192 190 187 166   0; ...
     nan   0 209 202 202 202 202 190 190 190 193 190 188 185 165   0; ...
     nan   0 216 203 202 199 197 195 193 191 190 187 185 182 163   0; ...
     nan   0 216 200 199 196 195 192 190 189 187 184 182 180 160   0; ...
     nan   0 216 196 194 191 189 186 185 183 181 178 176 173 155   0; ...
     nan   0 216 190 188 185 184 181 180 177 175 172 170 167 148   0; ...
     nan   0 216 184 182 180 178 175 174 171 169 166 164 161 144   0; ...
     nan   0 194 163 161 158 156 155 153 150 148 146 144 142 141   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 124 124 124 124  88   0 nan nan nan nan nan nan nan nan; ...
     nan   0 140 111 110 109 109   0   0   0   0   0   0   0   0 163; ...
     nan   0 124  97  97  96  96  95  93  98  99  99  99  99  89   0; ...
     nan   0 124  87  87  86  86  72   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0   0   0 155 204 202 199 198 196 175   0; ...
     nan   0 202 202 202 202 202 202 207 177 176 172 170 168 149   0; ...
     nan   0 196 181 181 181 181 170 170 170 173 170 169 167 148   0; ...
     nan   0 204 182 181 178 177 175 173 171 170 168 166 164 146   0; ...
     nan   0 204 179 178 176 175 172 171 169 167 165 163 161 143   0; ...
     nan   0 204 174 172 170 169 166 165 163 162 159 157 155 138   0; ...
     nan   0 204 169 167 165 163 161 160 157 155 153 151 148 132   0; ...
     nan   0 204 163 161 159 157 155 153 151 150 147 145 143 127   0; ...
     nan   0 184 143 141 139 138 136 135 132 131 128 127 126 124   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan 255 255 255 255 255 255 255 nan nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 nan nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% Go up on level in the file system hierarchy.
function [im, alpha] = mtki_icon_dir_up

  im = cat(3, ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan 198   0   0   0   0 198 nan nan nan nan nan nan nan nan; ...
     nan 198   0 228 211 211 165   0 198 nan nan nan nan nan nan nan; ...
     nan   0 243 171 189 188 177 110   0   0   0   0   0   0   0 154; ...
     nan   0 230 245 230 230 230 228 230 226 226 225 225 223 196   0; ...
     nan   0 247 211 211 210 115 206 204 202 202 200 198 196 147   0; ...
     nan   0 240 211 211 116   0 114 204 202 202 200 198 196 147   0; ...
     nan   0 240 210 124   0   0   0 111 198 198 196 195 193 144   0; ...
     nan   0 239 206 204 202   0 198 196 195 195 193 191 189 142   0; ...
     nan   0 237 202 200 198   0 195 193 191 191 189 187 186 139   0; ...
     nan   0 236 198 196 195   0   0   0  55  85  99 139 183 139   0; ...
     nan   0 232 195 193 191 189 187 186 183 183 183 183 183 139   0; ...
     nan   0 198 144 142 142 140 139 139 139 139 139 139 139 100   0; ...
     nan 154   0   0   0   0   0   0   0   0   0   0   0   0   0 154; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan 198   0   0   0   0 198 nan nan nan nan nan nan nan nan; ...
     nan 198   0 229 213 213 167   0 198 nan nan nan nan nan nan nan; ...
     nan   0 244 174 192 190 180 113   0   0   0   0   0   0   0 154; ...
     nan   0 231 246 231 231 231 229 231 228 228 227 227 225 198   0; ...
     nan   0 247 214 214 212 116 209 206 205 205 203 201 200 150   0; ...
     nan   0 240 214 214 118   0 116 206 205 205 203 201 200 150   0; ...
     nan   0 241 212 126   0   0   0 113 201 201 200 198 196 148   0; ...
     nan   0 240 209 206 205   0 201 200 198 198 196 194 193 145   0; ...
     nan   0 238 205 203 201   0 198 196 194 194 193 190 189 143   0; ...
     nan   0 238 201 200 198   0   0   0  56  86 100 141 186 143   0; ...
     nan   0 232 198 196 194 193 190 189 186 186 186 186 186 143   0; ...
     nan   0 201 148 146 145 144 143 143 143 143 143 143 143 103   0; ...
     nan 154   0   0   0   0   0   0   0   0   0   0   0   0   0 154; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan 198   0   0   0   0 198 nan nan nan nan nan nan nan nan; ...
     nan 198   0 222 202 202 151   0 198 nan nan nan nan nan nan nan; ...
     nan   0 240 153 173 170 159  92   0   0   0   0   0   0   0 154; ...
     nan   0 221 243 221 221 221 219 221 217 217 216 216 214 182   0; ...
     nan   0 246 197 196 194 106 189 186 185 185 182 179 177 124   0; ...
     nan   0 235 197 196 108   0 105 186 185 185 182 179 177 124   0; ...
     nan   0 236 194 115   0   0   0 101 179 179 177 175 173 120   0; ...
     nan   0 235 189 186 185   0 179 177 175 175 173 171 168 117   0; ...
     nan   0 232 185 182 179   0 175 173 171 171 168 165 163 114   0; ...
     nan   0 231 179 177 175   0   0   0  49  75  86 121 160 114   0; ...
     nan   0 224 175 173 171 168 165 163 160 160 160 160 160 114   0; ...
     nan   0 186 120 118 117 115 114 114 114 114 114 114 114  81   0; ...
     nan 154   0   0   0   0   0   0   0   0   0   0   0   0   0 154; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan; ...
     nan nan 255 255 255 255 255 255 nan nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 255 nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% Home directory.
function [im, alpha] = mtki_icon_dir_home

  im = cat(3, ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 142 142 142 142 102   0 nan nan nan nan nan nan nan nan; ...
     nan   0 161 132 132 131 130   0   0   0   0   0   0   0   0 163; ...
     nan   0 142 116 115 115 114 113 111 114 114 114 114 114 102   0; ...
     nan   0 142 104 104 103  82  68   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0  43 105 180 216 214 210 209 208 186   0; ...
     nan   0 214 214 214 214 214 214 145 145 193 189 187 185 165   0; ...
     nan   0 208 200 200 200 200 145 237 237 145 188 186 184 164   0; ...
     nan   0 215 201 200 197 145 237 237 237 237 145 184 181 161   0; ...
     nan   0 215 198 197 145 237 237 237 237 237 237 145 179 158   0; ...
     nan   0 215 193 191 188 145 237 237 237 237 145 175 172 153   0; ...
     nan   0 215 188 186 183 145 237 237 237 237 145 169 165 147   0; ...
     nan   0 215 182 180 178 145 145 145 145 145 145 163 159 142   0; ...
     nan   0 193 161 159 156 155 153 151 148 147 145 143 140 139   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 146 146 146 146 105   0 nan nan nan nan nan nan nan nan; ...
     nan   0 164 135 134 133 132   0   0   0   0   0   0   0   0 163; ...
     nan   0 146 119 119 118 117 115 114 114 117 117 117 117 105   0; ...
     nan   0 146 106 106 105  84  70   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0  43 106 181 217 215 212 211 209 187   0; ...
     nan   0 215 215 215 215 215 215 146 146 195 192 190 187 166   0; ...
     nan   0 209 202 202 202 202 146 238 238 146 190 188 185 165   0; ...
     nan   0 216 203 202 199 146 238 238 238 238 146 185 182 163   0; ...
     nan   0 216 200 199 146 238 238 238 238 238 238 146 180 160   0; ...
     nan   0 216 196 194 191 146 238 238 238 238 146 176 173 155   0; ...
     nan   0 216 190 188 185 146 238 238 238 238 146 170 167 148   0; ...
     nan   0 216 184 182 180 146 146 146 146 146 146 164 161 144   0; ...
     nan   0 194 163 161 158 156 155 153 150 148 146 144 142 141   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan 171   0   0   0   0   0 176 nan nan nan nan nan nan nan nan; ...
     nan   0 124 124 124 124  88   0 nan nan nan nan nan nan nan nan; ...
     nan   0 140 111 110 109 109   0   0   0   0   0   0   0   0 163; ...
     nan   0 124  97  97  96  96  95  93  98  99  99  99  99  89   0; ...
     nan   0 124  87  87  86  68  57   0   0   0   0   0   0   0   0; ...
     nan   0   0   0   0   0  40 100 169 204 202 199 198 196 175   0; ...
     nan   0 202 202 202 202 202 202 128 128 176 172 170 168 149   0; ...
     nan   0 196 181 181 181 181 128 226 226 128 170 169 167 148   0; ...
     nan   0 204 182 181 178 128 226 226 226 226 128 166 164 146   0; ...
     nan   0 204 179 178 128 226 226 226 226 226 226 128 161 143   0; ...
     nan   0 204 174 172 170 128 226 226 226 226 128 157 155 138   0; ...
     nan   0 204 169 167 165 128 226 226 226 226 128 151 148 132   0; ...
     nan   0 204 163 161 159 128 128 128 128 128 128 145 143 127   0; ...
     nan   0 184 143 141 139 138 136 135 132 131 128 127 126 124   0; ...
     nan 163   0   0   0   0   0   0   0   0   0   0   0   0   0 163; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan 255 255 255 255 255 255 255 nan nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 nan nan nan nan nan nan nan nan; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% New directory.
function [im, alpha] = mtki_icon_dir_new

  im = cat(3, ...
    [nan nan nan nan nan nan nan nan nan   1   5   6   6   5   1 nan; ...
     nan nan nan nan nan nan nan nan   4  20  47  61  61  47  20   4; ...
     nan nan   0   0   0   0   0   2  21  75 142 170 170 142  75  21; ...
     nan   0   0 228 211 211 165   6  49 142 219 242 242 220 142  49; ...
     nan   0 243 171 189 188 177 114  61 175 238 251 251 239 175  63; ...
     nan   0 230 245 230 230 230 228 230 234 243 251 251 243 227  62; ...
     nan   0 247 211 211 210 208 207 210 223 238 247 247 237 200  48; ...
     nan   0 240 211 211 210 208 206 206 211 223 222 222 220 174  20; ...
     nan   0 240 210 208 206 204 202 200 201 205 205 204 201 151   4; ...
     nan   0 239 206 204 202 200 198 196 195 195 193 192 189 142   0; ...
     nan   0 237 202 200 198 196 195 193 191 191 189 187 186 139   0; ...
     nan   0 236 198 196 195 193 191 189 187 187 186 183 183 139   0; ...
     nan   0 232 195 193 191 189 187 186 183 183 183 183 183 139   0; ...
     nan   0 198 144 142 142 140 139 139 139 139 139 139 139 100   0; ...
     nan   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan   1   4   5   5   4   1 nan; ...
     nan nan nan nan nan nan nan nan   3  18  42  55  55  42  18   3; ...
     nan nan   0   0   0   0   0   2  19  67 127 155 155 127  67  19; ...
     nan   0   0 229 213 213 167   4  43 127 204 234 234 206 128  44; ...
     nan   0 244 174 192 190 180 116  56 159 230 250 250 231 160  56; ...
     nan   0 231 246 231 231 231 228 225 220 235 250 250 235 212  56; ...
     nan   0 247 214 214 212 210 209 207 210 224 238 239 224 187  43; ...
     nan   0 240 214 214 212 210 209 206 206 210 209 209 208 169  18; ...
     nan   0 241 212 210 209 206 205 202 201 202 203 201 198 153   2; ...
     nan   0 240 209 206 205 203 201 200 197 198 195 193 193 144   0; ...
     nan   0 238 205 203 201 200 198 196 194 194 193 190 189 143   0; ...
     nan   0 238 201 200 198 196 194 193 190 190 189 186 186 143   0; ...
     nan   0 232 198 196 194 193 190 189 186 186 186 186 186 143   0; ...
     nan   0 201 148 146 145 144 143 143 143 143 143 143 143 103   0; ...
     nan   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan], ...
    [nan nan nan nan nan nan nan nan nan   0   2   2   2   2   0 nan; ...
     nan nan nan nan nan nan nan nan   1  10  25  32  32  25  10   2; ...
     nan nan   0   0   0   0   0   1  11  40  79 100 100  79  40  11; ...
     nan   0   0 222 202 202 151   2  25  79 155 205 205 155  80  26; ...
     nan   0 240 153 173 170 159  93  32 106 202 246 246 202 106  32; ...
     nan   0 221 243 221 221 221 216 194 163 207 246 246 207 153  32; ...
     nan   0 246 197 196 194 192 187 172 153 172 208 208 170 128  25; ...
     nan   0 235 197 196 194 192 188 180 165 153 148 147 150 124  10; ...
     nan   0 236 194 192 189 186 185 180 172 167 162 160 162 119   1; ...
     nan   0 235 189 186 185 182 179 177 173 172 169 167 166 116   0; ...
     nan   0 232 185 182 179 177 175 173 171 171 168 165 163 114   0; ...
     nan   0 231 179 177 175 173 171 168 165 165 163 160 160 114   0; ...
     nan   0 224 175 173 171 168 165 163 160 160 160 160 160 114   0; ...
     nan   0 186 120 118 117 115 114 114 114 114 114 114 114  81   0; ...
     nan   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

  if nargout == 2
    im = uint8(im);
  end

  alpha = uint8( ...
    [nan nan nan nan nan nan nan nan nan 255 255 255 255 255 255 nan; ...
     nan nan nan nan nan nan nan nan 255 255 255 255 255 255 255 255; ...
     nan nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255; ...
     nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan nan]);

% mtk/message
%
%      mtk('message');
%      mtk('message', string)
%      mtk('message', title, string)
%
% Create a modal message dialog box.
%
% Optional argument STRING defaults to the literal string 'No news are
% good news'.  If STRING is empty, no dialog box is created.  Optional
% argument TITLE defaults to the literal string 'Message'.
function mtk_message(varargin)

  switch nargin
   case 0
    t = 'Message';
    s = 'No news are good news';
   case 1
    t = 'Message';
    s = varargin{1};
   case 2
    t = varargin{1};
    s = varargin{2};
   otherwise
    error(nargchk(0, 2, nargin));
  end

  s = mtki_wash(s, '-period');
  if isempty(s)
    return;
  end

  uiwait(msgbox(s, t, 'modal'));

% mtk/warning
%
%      mtk('warning')
%      mtk('warning', string)
%      mtk('warning', title, string)
%
% Create a modal warning message dialog box.
%
% Optional argument STRING defaults to the value of the 'lastwarn'
% function.  If STRING is empty, no dialog box is created.  Optional
% argument TITLE defaults to the literal string 'Warning'.
function mtk_warning(varargin)

  switch nargin
   case 0
    t = 'Warning';
    s = lastwarn;
   case 1
    t = 'Warning';
    s = varargin{1};
   case 2
    t = varargin{1};
    s = varargin{2};
   otherwise
    error(nargchk(0, 2, nargin));
  end

  s = mtki_wash(s, '-error', '-period');
  if isempty(s)
    return;
  end

  uiwait(warndlg(s, t, 'modal'));

% mtk/error
%
%      mtk('error')
%      mtk('error', string)
%      mtk('error', title, string)
%
% Create a modal error message dialog box.
%
% Optional argument STRING defaults to the value of the 'lasterr'
% function.  If STRING is empty, no dialog box is created.  Optional
% argument TITLE defaults to the literal string 'Error'.
function mtk_error(varargin)

  switch nargin
   case 0
    t = 'Error';
    s = lasterr;
   case 1
    t = 'Error';
    s = varargin{1};
   case 2
    t = varargin{1};
    s = varargin{2};
   otherwise
    error(nargchk(0, 2, nargin));
  end

  s = mtki_wash(s, '-error', '-period');
  if isempty(s)
    return;
  end

  uiwait(errordlg(s, t, 'modal'));

% mtk/question
%
%      p = mtk('question')
%      p = mtk('question', string)
%      p = mtk('question', title, string)
%
% Create a modal yes/no question message dialog box.
%
% Optional argument STRING defaults to the literal string 'Are you
% experienced?'.  If STRING is empty, no dialog box is created.
% Optional argument TITLE defaults to the literal string 'Question'.
%
% Return value is non-zero if the user responds with yes.
function p = mtk_question(varargin)

  switch nargin
   case 0
    t = 'Question';
    s = 'Are you experienced?';
   case 1
    t = 'Question';
    s = varargin{1};
   case 2
    t = varargin{1};
    s = varargin{2};
   otherwise
    error(nargchk(0, 2, nargin));
  end

  s = mtki_wash(s, '-period');
  if isempty(s)
    return;
  end

  switch questdlg(s, t, 'Yes', 'No', 'Yes')
   case 'Yes'
    p = 1;
   otherwise
    p = 0;
  end

% mtk/getfile
%
%      file = mtk('getfile', ...)
%
% Open a file selection dialog box and return the full file name
% selected by the user.  Return value FILE is empty if the user
% cancels the dialog box.
%
% See 'uigetfile', for more information about the arguments.
function file = mtk_getfile(varargin)

  [base, dir] = uigetfile(varargin{:});
  if ischar(base)
    file = fullfile(dir, base);
  else
    file = [];
  end

% mtk/putfile
%
%      file = mtk('putfile', ...)
%
% Open a file selection dialog box and return the full file name
% selected by the user.  Return value FILE is empty if the user
% cancels the dialog box.
%
% See 'uiputfile', for more information about the arguments.
function file = mtk_putfile(varargin)

  [base, dir] = uiputfile(varargin{:});
  if ischar(base)
    file = fullfile(dir, base);
  else
    file = [];
  end

% mtk/filesel
%
%      file = mtk('filesel', ...)
%
% Open a file selection dialog box and return the full file name
% selected by the user.  Return value FILE is empty if the user
% cancels the dialog box.
%
% Remaining arguments are property/value pairs.
%
% Style
%      Dialog box style.  A value of 'open' (reading) or 'save' (writing)
%      creates a file selection dialog box.  A value of 'dir' creates a
%      directory selection dialog box.  Default value is 'open'.
%
% SelectionType
%      File selection type.  Value is either 'single' or 'multiple'.
%      Default is to select only a single file.
%
% MustExist
%      Whether the selected file must exist.  Value is either 'on' or
%      'off'.  Default is 'on' when selecting a file name for reading
%      and 'off' when selection a file name for writing.
%
% WindowTitle
%      The window title (a string).  If no window title is specified, a
%      default title according to the dialog box style is used.
%
% HomeDirectory
%      The home directory for this dialog box.  Default is the current
%      working directory.
%
% Directory
%      The initial directory for this dialog box.  Default is the current
%      working directory.
%
% DefaultFileName
%      The default value for the selected file name.  If the default file
%      name has a directory part, this directory part replaces the value
%      of the 'Directory' property.
%
% Filter
%      A list of file filters.  Value is a cell array of strings with
%      two columns.  The first column contains the globbing pattern
%      and the second column contains a description.  An empty pattern
%      represents all files.  If no entry with an empty pattern exists,
%      it will be appended automatically.  Default is to list all files.
%
% FilterIndex
%      The index (one based) of the initially active file filter.  Default
%      is the first file filter.
%
% When selecting a directory, the properties 'SelectionType', 'MustExist',
% 'DefaultFilename', 'Filter', and 'FilterIndex' can not be overwritten by
% the user.
function filename = mtk_filesel(varargin)

  style = 'open';
  multi = 0;
  new = [];
  title = [];
  home = [];
  path = [];
  name = {};
  filter = [];
  index = 1;

  if rem(nargin, 2) ~= 0
    error('Wrong number of arguments');
  end

  for i = 1:2:nargin
    prop = varargin{i};
    val = varargin{i + 1};
    switch lower(prop)
     case 'style'
      if ischar(val)
        switch val
         case {'open', 'save', 'dir'}
          style = val;
         otherwise
          error('Invalid argument');
        end
      else
        error('Invalid argument');
      end
     case 'selectiontype'
      if ischar(val)
        switch val
         case 'single'
          multi = 0;
         case 'multiple'
          multi = 1;
         otherwise
          error('Invalid argument');
        end
      else
        error('Invalid argument');
      end
     case 'mustexist'
      if ischar(val)
        switch val
         case 'on'
          new = 0;
         case 'off'
          new = 1;
         otherwise
          error('Invalid argument');
        end
      else
        error('Invalid argument');
      end
     case 'windowtitle'
      if isempty(val)
        title = [];
      elseif ischar(val)
        title = val;
      else
        error('Invalid argument');
      end
     case 'homedirectory'
      if isempty(val)
        home = [];
      elseif ischar(val)
        home = val;
      else
        error('Invalid argument');
      end
     case 'directory'
      if isempty(val)
        path = [];
      elseif ischar(val)
        path = val;
      else
        error('Invalid argument');
      end
     case 'defaultfilename'
      if isempty(val)
        name = {};
      elseif ischar(val)
        [dir, base] = mtki_fileparts(val);
        if ~ isempty(dir)
          path = dir;
        end
        name = {base};
      else
        error('Invalid argument');
      end
     case 'filter'
      if isempty(val)
        filter = [];
      elseif iscellstr(val) & size(val, 2) == 2
        filter = val;
      else
        error('Invalid argument');
      end
     case 'filterindex'
      if isnumeric(val) & isreal(val) & ~ issparse(val) & numel(val) == 1 & rem(val, 1) == 0 & val > 0
        index = val;
      else
        error('Invalid argument');
      end
    end
  end

  switch style
   case 'open'
    if isempty(new)
      new = 0;
    end
   case 'save'
    if isempty(new)
      new = 1;
    end
   case 'dir'
    % These options have no effect
    % when selecting a directory.
    multi = 0;
    new = 0;
    name = {};
    filter = [];
    index = 1;
  end

  % Window title.
  if isempty(title)
    switch style
     case 'open'
      title = 'Open File';
     case 'save'
      title = 'Save File';
     case 'dir'
      title = 'Select Directory';
    end
  end

  % Home directory.
  tem = mtki_dirpath(home);
  if isempty(tem)
    if exist(home, 'dir')
      error(sprintf('%s: access denied', home));
    else
      error(sprintf('%s: no such directory', home));
    end
  else
    home = tem;
  end

  % Initial directory.
  tem = mtki_dirpath(path);
  if isempty(tem)
    if exist(path, 'dir')
      error(sprintf('%s: access denied', path));
    else
      error(sprintf('%s: no such directory', path));
    end
  else
    path = tem;
  end

  % File filters.
  if isempty(filter)
    filter = {[], 'All files'};
  end
  if ~ iscell(filter) | size(filter, 2) ~= 2
    error('Invalid filter argument');
  end
  ind = ~ cellfun('isempty', filter(:, 1));
  if all(ind)
    filter(end + 1, :) = {[], 'All files'};
  end

  if index > size(filter, 1)
    error('Invalid filter index');
  end

  % File filter menu labels.
  filters = filter(:, 2);
  filters(ind) = strcat(filters(ind), ' (', filter(ind, 1), ')');

  % Commit button label.
  switch style
   case 'open'
    commit = 'Open';
   case 'save'
    commit = 'Save';
   case 'dir'
    commit = 'Select';
  end

  % Create figure window.
  fig = mtk('figure', 0, '0x0', ...
            'Name', title, ...
            'WindowStyle', 'modal', ...
            'Visible', 'off');

  mtk('render', fig, ...
      {'box', ...
       'Margin', 16, ...
       'Padding', 16, ...
       {'box', ...
        'Orientation', 'vertical', ...
        'Padding', 8, ...
        {'box', ...
         'Orientation', 'vertical', ...
         {'text', ...
          'String', 'Path:'}, ...
         {'box', ...
          'Padding', 8, ...
          {'edit', ...
           'Flexibility', 1, ...
           'String', '', ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'path'}, ...
          {'pushbutton', ...
           'Width', 24, ...
           'Height', 24, ...
           'CData', mtk('icon', 'up', fig), ...
           'TooltipString', 'Go up one level', ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'up'}, ...
          {'pushbutton', ...
           'Width', 24, ...
           'Height', 24, ...
           'CData', mtk('icon', 'home', fig), ...
           'TooltipString', 'Go back home', ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'home'}, ...
          {'pushbutton', ...
           'Width', 24, ...
           'Height', 24, ...
           'CData', mtk('icon', 'dir', fig), ...
           'TooltipString', 'Create new directory', ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'new'}}}, ...
        {'box', ...
         'Padding', 8, ...
         {'box', ...
          {'text', ...
           'String', 'Directories:'}, ...
          {'listbox', ...
           'Width', 192, ...
           'Height', 256, ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'dir'}}, ...
         {'box', ...
          {'text', ...
           'String', 'Files:'}, ...
          {'listbox', ...
           'Width', 192, ...
           'Height', 256, ...
           'Max', 1 + multi, ...
           'Callback', {@mtki_filesel_handler}, ...
           'Tag', 'file'}}}, ...
        {'box', ...
         'Alignment', 'baseline', ...
         {'text', ...
          'String', 'Selection:'}, ...
         {'edit', ...
          'Flexibility', 1, ...
          'String', '', ...
          'Max', 1 + multi, ...
          'Callback', {@mtki_filesel_handler}, ...
          'Tag', 'name'}}, ...
        {'box', ...
         'Alignment', 'baseline', ...
         {'text', ...
          'Width', 'Selection:', ...
          'String', 'Filter:'}, ...
         {'popupmenu', ...
          'Width', 1, ...
          'Flexibility', 1, ...
          'String', filters, ...
          'Value', index, ...
          'UserData', filter(:, 1), ...
          'Callback', {@mtki_filesel_handler}, ...
          'Tag', 'glob'}}}, ...
       {'box', ...
        'Padding', 8, ...
        {'pushbutton', ...
         'Width', 80, ...
         'String', commit, ...
         'Callback', {@mtki_filesel_handler}, ...
         'Tag', 'commit'}, ...
        {'pushbutton', ...
         'Width', 80, ...
         'String', 'Cancel', ...
         'Callback', {@mtki_filesel_handler}, ...
         'Tag', 'cancel'}}});

  % Global variables.  An empty path forces the mtki_filesel_update()
  % procedure to fill out the file selection dialog box.
  env.fig = fig;
  env.style = style;
  env.multi = multi;
  env.exist = ~ new;
  env.home = home;
  env.path = [];
  env.name = name;
  env.glob = filter{index, 1};
  env.h = guihandles(fig);

  if strcmp(style, 'dir')
    set(env.h.glob, 'Enable', 'off');
  end

  set(fig, 'UserData', env);
  mtki_filesel_update(env, path);

  set(fig, 'Visible', 'on');
  mtk('focus', env.h.name);

  uiwait(fig);

  if ishandle(fig)
    env = get(fig, 'UserData');
    % Delete figure window.
    delete(fig);

    if multi
      filename = env.name;
    else
      filename = env.name{1};
    end
  else
    filename = [];
  end

% File selection dialog box call-back handler.
function mtki_filesel_handler(obj, event)

  fig = get(obj, 'Parent');
  env = get(fig, 'UserData');

  dirp = strcmp(env.style, 'dir');
  filep = ~ dirp;

  switch get(obj, 'Tag')
   case 'path'
    str = get(obj, 'String');
    if filep
      [dir, base] = mtki_fileparts(str);
      if strfind(base, '*')
        mtki_filesel_update(env, dir, base);
      else
        mtki_filesel_update(env, str);
      end
    else
      mtki_filesel_update(env, str);
    end
   case 'up'
    mtki_filesel_update(env, fullfile(env.path, '..'));
   case 'home'
    mtki_filesel_update(env, env.home);
   case 'new'
    str = mtki_filesel_input('Directory name:');
    if isempty(str)
      return;
    end
    [dir, base] = mtki_fileparts(str);
    if ~ isempty(dir)
      mtk_error('Invalid directory name.');
    elseif mkdir(env.path, str) == 0
      mtk_error('Can not create directory.');
    else
      % Visit newly created directory.
      mtki_filesel_update(env, fullfile(env.path, str));
    end
   case 'dir'
    str = get(obj, 'String');
    val = get(obj, 'Value');
    switch get(fig, 'SelectionType')
     case 'normal'
      if dirp
        set(env.h.name, 'String', str(val));
      end
     case 'open'
      mtki_filesel_update(env, fullfile(env.path, str{val}));
    end
   case 'file'
    if filep
      str = get(obj, 'String');
      val = get(obj, 'Value');
      switch get(fig, 'SelectionType')
       case 'normal'
        set(env.h.name, 'String', str(val));
        % Clear default file name.
        env.name = {};
        set(fig, 'UserData', env);
      end
    end
   case 'name'
    if filep
      if ~ isempty(env.name)
        env.name = get(obj, 'String');
        set(fig, 'UserData', env);
      end
    end
   case 'glob'
    if filep
      tem = get(obj, 'UserData');
      glob = tem{get(obj, 'Value')};
      mtki_filesel_update(env, env.path, glob);
    end
   case 'commit'
    str = get(env.h.name, 'String');
    str(cellfun('isempty', str)) = [];
    if isempty(str) & filep
      mtk_error('Please select a file name.');
    else
      path = env.path;
      if isempty(str)
        name = {path};
      else
        name = cell(size(str));
        for i = 1:numel(name)
          name{i} = fullfile(path, str{i});
        end
      end
      p = ones(size(name));
      if env.exist
        if dirp
          for i = 1:numel(name)
            p(i) = exist(name{i}, 'dir');
          end
        else
          % Caveat, exist('/foo', 'file') returns 2 if '/foo' does not
          % exist but '/foo.m' does.
          [d, f] = mtki_filesel_list(env.path, []);
          for i = 1:numel(name)
            switch str{i}
             case f
             otherwise
              p(i) = 0;
            end
          end
        end
      end
      if p
        env.name = name;
        set(fig, 'UserData', env);
        uiresume(fig);
      else
        if dirp
          mtk_error('Directory does not exist.');
        else
          ind = find(p == 0);
          switch numel(ind)
           case 1
            mtk_error(sprintf('File ''%s'' does not exist.', str{ind}));
           case 2
            mtk_error(sprintf('Files ''%s'' and ''%s'' do not exist.', str{ind(1)}, str{ind(2)}));
           otherwise
            tem = str{ind(1)};
            for i = 2:(numel(ind) - 1)
              tem = [tem, ''', ''', str{ind(i)}];
            end
            tem = [tem, ''', and ''', str{ind(end)}];
            mtk_error(sprintf('Files ''%s'' do not exist.', tem));
          end
        end
      end
    end
   case 'cancel'
    delete(fig);
  end

% Update file selection dialog box.
function mtki_filesel_update(env, path, glob)

  if nargin < 3
    glob = env.glob;
  end

  new_path = ~ strcmp(path, env.path);
  new_glob = ~ strcmp(glob, env.glob);

  if new_path
    tem = mtki_dirpath(path);
    if isempty(tem)
      if exist(path, 'dir')
        mtk_error('Access denied.');
      else
        mtk_error('No such directory.');
      end
      % No change.
      new_path = 0;
    else
      % Save canonical path.
      env.path = tem;
    end
    % Update/restore path.
    set(env.h.path, 'String', env.path);
  end

  if new_glob
    env.glob = glob;
  end

  if new_path | new_glob
    [d, f] = mtki_filesel_list(env.path, env.glob);
    set(env.h.dir, 'String', d, 'Value', 1);
    set(env.h.file, 'String', f, 'Value', 1);
    set(env.h.name, 'String', env.name);
    set(env.fig, 'UserData', env);
  end

% Return a directory and file listing.
function [d, f] = mtki_filesel_list(path, glob)

  dirent = dir(path);
  isdir = logical([dirent.isdir]);
  d = {dirent(isdir).name};
  d(strcmp(d, '.')) = [];

  if isempty(glob) | strcmp(glob, '*')
    f = {dirent(~ isdir).name};
  else
    dirent = dir(fullfile(path, glob));
    isdir = logical([dirent.isdir]);
    f = {dirent(~ isdir).name};
  end

% File selection input dialog.
function str = mtki_filesel_input(prompt)

  % Create figure window.
  fig = mtk('figure', 0, '0x0', ...
            'Name', 'Input', ...
            'WindowStyle', 'modal', ...
            'Visible', 'off');

  mtk('render', fig, ...
      {'box', ...
       'Margin', 16, ...
       'Padding', 16, ...
       {'box', ...
        'Orientation', 'vertical', ...
        {'text', ...
         'String', prompt}, ...
        {'edit', ...
         'Flexibility', 1, ...
         'Tag', 'input'}}, ...
       {'box', ...
        'Padding', 8, ...
        {'pushbutton', ...
         'Width', 80, ...
         'String', 'OK', ...
         'Callback', 'uiresume(gcf);'}, ...
        {'pushbutton', ...
         'Width', 80, ...
         'String', 'Cancel', ...
         'Callback', 'delete(gcf);'}}});

  h = guihandles(fig);

  set(fig, 'Visible', 'on');
  mtk('focus', h.input);

  uiwait(fig);

  if ishandle(fig)
    str = get(h.input, 'String');
    % Delete figure window.
    delete(fig);
  else
    str = [];
  end

% mtk/caller
%
%      filename = mtk('caller')
%
% Return the caller's full file name.
%
% If called from the base workspace, the return value is empty.
function filename = mtk_caller(level)

  if nargin < 1
    level = 0;
  end

  tem = dbstack;
  level = 3 + level;

  if numel(tem) >= level
    filename = tem(level).name;
  else
    filename = [];
  end

% mtk/load
%
%      val = mtk('load', filename)
%
% Evaluate the contents of a text file as a Matlab expression.
%
% First argument FILENAME specifies the file to be read.
%
% Within the file, empty lines and lines containing only spaces and
% Matlab comments are ignored, and newline characters are treated as
% horizontal space characters.
function val = mtk_load(filename)

  str = mtk_read(filename);
  if isempty(str)
    val = [];
  else
    val = evalin('base', str);
  end

% mtk/read
%
%      str = mtk('read', filename)
%
% Read a MTk layout specification.
%
% First argument FILENAME specifies the file to be read.
function str = mtk_read(filename)

  str = mtki_wash(fileread(filename), '-eol', '-final');

  % Remove empty lines and Matlab comments.
  nl = sprintf('\n');

  eol = strfind(str, nl);
  bol = [0, eol] + 1;
  bol(end) = [];

  for i = fliplr(1:numel(eol))
    ind = bol(i):eol(i);
    sp = isspace(str(ind));
    other = find(~ sp);
    if sp | str(bol(i) + other(1) - 1) == '%'
      str(ind) = [];
    end
  end

  % Replace newline characters with spaces.
  ind = strfind(str, nl);
  if ind
    str(ind) = ' ';
  end

% mtk/render
%
%      mtk('render', f, spec, ...)
%
% The 'render' command parses a widget layout and creates these widgets
% in the figure F.  After that, the widgets are laid out intrinsically,
% i.e., at their minimum size.  If the figure can be resized, a resize
% function is installed to maintain the widget layout.
%
% First argument F is a figure handle.
% Second argument SPEC is a layout element.
%
% If the 'render' command is called with more than two arguments, or
% if the second argument SPEC does not look like a layout element, the
% arguments are interpreted as property/value pairs and child elements
% of a box whose default orientation is vertical.
%
% A layout element is a cell array whose first element is the name of the
% layout element (a string).  There are three types of layout elements.
%
%   1. Top-level layout elements have no physical representation.
%      They are only used to position other layout elements.
%
%   2. Meta-widget layout elements are combinations of multiple other
%      layout elements providing a special functionality.
%
%   3. Base widget layout elements represent a single widget.
%
% Top level layout elements:
%
% box
%      A box is a container element which can contain any number of child
%      elements.  Child elements are placed consecutively in the order they
%      are specified.  Remaining cell array elements are property/value
%      pairs and any number of child elements.
%
%      See 'mtk/box', for more details about the box element.
%
% grid
%      A grid is a container element with any number of rows and columns.
%
%      See 'mtk/grid', for more details about the grid element.  [TODO]
%
% Meta-widget layout elements:
%
% group
%      A group is a frame widget with a text label at its top edge.
%
%      See 'mtk/group', for more details about the group element.
%
% tabs
%      A tab panel is a meta-widget that has one or more tabs along the
%      top edge of a content panel.  Selecting a tab means to display the
%      associated content in the panel.
%
%      See 'mtk/tabs', for more details about the tab panel element.
%
% Base widget layout elements:
%
% WIDGET
%      Any MTk widget, i.e., 'axes', 'frame', 'pushbutton', 'togglebutton',
%      'radiobutton', 'checkbox', 'popupmenu' 'edit', 'slider', 'text', or
%      'listbox'.  Arguments are property/value pairs.  All widget elements
%      accept these additional element properties:
%
%      Flexibility
%           A widget becomes flexible when the flexibility factor (a non-
%           negative floating point value) is greater than zero.  Flexible
%           widgets can shrink or grow as the parent shrinks and grows.
%           When multiple children of a parent are flexible, any extra
%           space is divided among the flexible children equal to the
%           ratio of flexibility that it is responsible for.
%
%           The default value is zero.
%
%      Width
%      Height
%           For inflexible widgets, the 'Width' and 'Height' properties can
%           be used to specify the size of the widget.  If these properties
%           are omitted, an inflexible widget will be sized intrinsically,
%           i.e., it will be given the minimum amount of space required to
%           lay out the widget.
%
%           For both properties, the default value is zero.
%
% The widget layout is loosely modeled after the XML User Interface
% Language (XUL).  See <http://www.mozilla.org/projects/xul/xul.html>,
% for more details.
%
% Caveats:
%
%   1. The figure F should be invisible to increase performance and
%      avoid flicker.
%
%   2. The figure's main menu should be in place before the 'render'
%      command is called.
function mtk_render(f, spec, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Save existing object handles.
  obj = findobj(f);

  % Render widgets.
  try
    if nargin == 2 & iscell(spec)
      layout = mtki_render_top_level(f, 'vertical', spec);
    else
      layout = mtki_render(f, 'vertical', {'box', spec, varargin{:}});
    end
  catch
    % Delete created widgets.
    delete(setdiff(findobj(f), obj));
    % Signal error.
    error(lasterr);
  end

  % Set intrinsic figure dimensions.
  width = layout.width;
  height = layout.height;

  pos = get(f, 'Position');
  pos(3:4) = [width height];
  set(f, 'Position', pos);

  % Layout widgets.
  mtki_layout(f, layout, 0, 0, width, height);

  % Save layout data.
  setappdata(f, 'mtk_layout', layout);

  % Install resize function.
  switch get(f, 'Resize')
   case 'on'
    set(f, 'ResizeFcn', @mtki_resize);
  end

  % Display figure window.
  movegui(f, 'onscreen');
  set(f, 'Visible', 'on');

% Render a top-level element.
function layout = mtki_render_top_level(f, orient, spec)

  layout = mtki_render(f, orient, spec);
  switch layout.elem
   case 'box'
   case 'grid'
   otherwise
    error('Invalid top-level layout element');
  end

% Render an element.
function layout = mtki_render(f, orient, spec)

  switch spec{1}
   case {'box', 'hbox', 'vbox'}
    layout = mtki_render_box(f, orient, spec);
   case {'grid', 'hgrid', 'vgrid'}
    layout = mtki_render_grid(f, spec);
   case 'group'
    layout = mtki_render_group(f, spec);
   case 'tabs'
    layout = mtki_render_tabs(f, spec);
   case {'axes', ...
         'frame', ...
         'pushbutton', ...
         'togglebutton', ...
         'radiobutton', ...
         'checkbox', ...
         'popupmenu', ...
         'edit', ...
         'slider', ...
         'text', ...
         'listbox'}
    layout = mtki_render_widget(f, spec);
   otherwise
    error('Unknown layout element');
  end

% mtk/box
%
% The MTk box element is one of the fundamental elements to lay out child
% elements.  If the box has a horizontal orientation, the child elements
% are laid out from left to right in the order they are specified.  If
% the box has vertical orientation, the child elements are laid out from
% top to bottom.  Child elements do not overlap.  A box element has the
% following properties:
%
% Orientation
%      Specifies whether the children of the box are oriented horizontally
%      or vertically.  Possible values are:
%
%      horizontal
%           Child elements of the box are placed next to each other in
%           a row in the order that they appear in the widget layout.
%      vertical
%           Child elements of the element are placed under each other in
%           a column in the order that they appear in the widget layout.
%      inherit
%           If a box is the child of a vertical box, it is oriented
%           horizontally.  Otherwise, if a box is the child of a
%           horizontal box, it is oriented vertically.
%
%      The default value is 'inherit' for invisible boxes and 'vertical' for
%      visible boxes.
%
% Direction
%      The direction in which the child elements of the box are placed.
%      Possible values are:
%
%      normal
%           The child elements are placed left to right or top to bottom in
%           the order they appear in the widget layout.
%      reverse
%           The child elements are placed right to left or bottom to top.
%           This is the reverse order in which they appear in the widget
%           layout.
%
%      The default value is 'normal'.
%
%      The box direction only affects the order of the children in the
%      widget layout, and not the alignment or packing of the children.
%
% Alignment
%      The alignment property specifies how child elements of the box
%      are laid out when they are smaller than the size of the box.
%      For horizontal boxes, it specifies how the children are aligned
%      vertically.  For vertical boxes, it specifies how the children
%      are aligned horizontally. Possible values are:
%
%      stretch
%           The child elements are stretched to fit the size of the
%           box.  For a horizontal box, the children are stretched to
%           the height of the box.  For a vertical box, the children
%           are stretched to the width of the box.  If the size of the
%           box changes, the children are stretched to fit.  Use the
%           'Flexibility' property to create elements that stretch in
%           the opposite direction.
%      start
%           Child elements are placed at the left or top edge of the
%           box.  If the box is larger than a child, the extra space is
%           placed at the right or bottom side.
%      center
%           Extra space is split equally along each side of the child
%           elements, resulting the children being placed in the center
%           of the box.
%      end
%           Child elements are placed at the right or bottom edge of the
%           box.  If the box is larger than a children, the extra space
%           is placed at the left or top side.
%
%      Vertical boxes accept these additional values:
%
%      left
%           Child elements are placed at the left edge of the box.  If
%           the box is wider than a child, the extra space is placed at
%           the right side.
%      right
%           Child elements are placed at the right edge of the box.  If
%           the box is wider than a children, the extra space is placed
%           at the left side.
%
%      Horizontal boxes accept these additional values:
%
%      top
%           Child elements are placed at the top edge of the box.  If the
%           box is higher than a child, the extra space is placed at the
%           bottom side.
%      bottom
%           Child elements are placed at the bottom edge of the box.  If
%           the box is higher than a child, the extra space is placed at
%           the top side.
%      baseline
%           Child elements are placed so that their text labels are lined
%           up.
%
%      The default value is 'stretch'.
%
%      When a value other than 'stretch' is used, the box's children are
%      no longer stretched.  They are sized intrinsically and then aligned
%      on the box axis according to the specified property value.
%
%      The 'Pack' property is related to the 'Alignment' property but is
%      used to specify the position in the opposite direction.
%
% Pack
%      This property specifies how child elements of the box are laid out
%      when they are smaller than the total size of the box.  For horizontal
%      boxes, it specifies how the children are placed horizontally.  For
%      vertical boxes, it specifies how the children are placed vertically.
%      Possible values are:
%
%      start
%           Child elements are placed at the left or top edge of the box.
%           If the box is larger than the total size of the children, the
%           extra space is placed at the right or bottom side.
%      center
%           Extra space is split equally along each side of the child
%           elements, resulting the children being placed in the center
%           of the box.
%      end
%           Child elements are placed at the right or bottom edge of the
%           box.  If the box is larger than the total size of the children,
%           the extra space is placed at the left or top side.
%      justify
%           Extra space is split equally along the gap between the child
%           elements.
%
%      Horizontal boxes accept these additional values:
%
%      left
%           Child elements are placed at the left edge of the box.  If the
%           box is wider than the total width of the children, the extra
%           space is placed at the right side.
%      right
%           Child elements are placed at the right edge of the box.  If the
%           box is wider than the total width of the children, the extra
%           space is placed at the left side.
%
%      Vertical boxes accept these additional values:
%
%      top
%           Child elements are placed at the top edge of the box.  If the
%           box is higher than the total hight of the children, the extra
%           space is placed at the bottom side.
%      bottom
%           Child elements are placed at the bottom edge of the box.  If
%           the box is higher than the total hight of the children, the
%           extra space is placed at the top side.
%
%      The default value is 'start'.
%
%      When all of the element within a box are inflexible or when all
%      elements have grown to their maximum sizes and can grow no further,
%      extra space may be left over in the box.  The 'Pack' property can
%      be used to dictate how any additional space should be distributed
%      between elements.
%
% Flexibility
%      A box becomes flexible when the flexibility factor (a non-negative
%      floating point value) is greater than zero.  Flexible boxes can
%      shrink or grow as the parent shrinks and grows.  When multiple
%      children of a parent are flexible, any extra space is divided among
%      the flexible children equal to the ratio of flexibility that it is
%      responsible for.
%
%      The default value is zero.
%
% Width
% Height
%      For inflexible objects, the 'Width' and 'Height' properties can
%      be used to specify the size of the box.  If these properties are
%      omitted, an inflexible box will be sized intrinsically, i.e., it
%      will be given the minimum amount of space required to lay out the
%      box.
%
%      For both properties, the default value is zero.
%
% Margin
%      Extra space to be added between the box edges and the children.
%      Value has to be an array with 1, 2, or 4 non-negative integral
%      numbers.
%
%      A scalar specifies the extra space to be added at the left edge,
%      right edge, top edge, and bottom edge.
%
%      The first element of a pair specifies the extra space to be added
%      at the left and right edge.  The second element specifies the extra
%      space to be added at the top and bottom edge.
%
%      If the value is an array with four elements, the elements specify
%      the extra space to be added at the left edge, right edge, top edge,
%      and bottom edge.
%
%      Default value is zero.
%
% Padding
%      Extra space to be added between the children of a box.
%      Value is a non-negative integral number.
%
%      Default value is zero.

% Render a box.
function layout = mtki_render_box(f, orient, spec)

  % Create layout structure.
  layout = mtki_layout_struct('box');

  % Parse property/value pairs.
  layout.prop = struct('orient', 'inherit', ...
                       'dir', 'normal', ...
                       'align', 'stretch', ...
                       'pack', 'start', ...
                       'width', 0, ...
                       'height', 0, ...
                       'margin', [0 0 0 0], ...
                       'pad', 0);

  switch spec{1}
   case 'hbox'
    layout.prop.orient = 'horizontal';
   case 'vbox'
    layout.prop.orient = 'vertical';
  end

  % Ignore first element.
  ind = [1];

  n = numel(spec);
  for i = 2:2:n
    arg = spec{i};
    if iscell(arg)
      break;
    end
    if i == n
      error('Incomplete property/value pair');
    end
    capture_arg = 0;
    val = spec{i + 1};
    switch lower(arg)
     case 'orientation'
      capture_arg = 1;
      switch val
       case {'inherit', ...
             'horizontal', ...
             'vertical'}
        layout.prop.orient = val;
       otherwise
        error('Invalid box orientation');
      end
     case 'direction'
      capture_arg = 1;
      switch val
       case {'normal', ...
             'reverse'}
        layout.prop.dir = val;
       otherwise
        error('Invalid box direction');
      end
     case 'alignment'
      capture_arg = 1;
      switch val
       case {'stretch', ...
             'start', 'center', 'end', ...
             'top', 'bottom', 'baseline', ...
             'left', 'right'}
        layout.prop.align = val;
       otherwise
        error('Invalid box alignment');
      end
     case 'pack'
      capture_arg = 1;
      switch val
       case {'justify', ...
             'start', 'center', 'end', ...
             'top', 'bottom', ...
             'left', 'right'}
        layout.prop.pack = val;
       otherwise
        error('Invalid box packing');
      end
     case 'flexibility'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & val >= 0
        layout.flex = val;
      else
        error('Invalid box flexibility');
      end
     case 'width'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.width = val;
      else
        error('Invalid box width');
      end
     case 'height'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.height = val;
      else
        error('Invalid box height');
      end
     case 'margin'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & any(numel(val) == [1 2 4]) & all(rem(val(:), 1) == 0) & all(val(:) >= 0)
        switch numel(val)
         case 1
          layout.prop.margin = repmat(val, 1, 4);
         case 2
          layout.prop.margin = [val(1) val(1) val(2) val(2)];
         otherwise
          layout.prop.margin = val(:)';
        end
      else
        error('Invalid box margin');
      end
     case 'padding'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.pad = val;
      else
        error('Invalid box padding');
      end
     otherwise
      error('Invalid box property');
    end
    % Save argument indices.
    if capture_arg
      ind = [ind, i, i + 1];
    end
  end

  if strcmp(layout.prop.orient, 'inherit')
    layout.prop.orient = orient;
  end

  switch layout.prop.orient
   case 'horizontal'
    switch layout.prop.align
     case 'top'
      layout.prop.align = 'start';
     case 'bottom'
      layout.prop.align = 'end';
     case {'left', 'right'}
      error('Invalid horizontal box alignment');
    end
    switch layout.prop.pack
     case 'left'
      layout.prop.pack = 'start';
     case 'right'
      layout.prop.pack = 'end';
     case {'top', 'bottom'}
      error('Invalid horizontal box packing');
    end
   case 'vertical'
    switch layout.prop.align
     case 'left'
      layout.prop.align = 'start';
     case 'right'
      layout.prop.align = 'end';
     case {'top', 'bottom', 'baseline'}
      error('Invalid vertical box alignment');
    end
    switch layout.prop.pack
     case 'top'
      layout.prop.pack = 'start';
     case 'bottom'
      layout.prop.pack = 'end';
     case {'left', 'right'}
      error('Invalid vertical box packing');
    end
  end

  % Render children.
  spec(ind) = [];

  n = numel(spec);
  if n > 0
    switch layout.prop.orient
     case 'horizontal'
      orient = 'vertical';
     case 'vertical'
      orient = 'horizontal';
    end
    layout.child = repmat(mtki_layout_struct([]), 1, n);
    for i = 1:n
      layout.child(i) = mtki_render(f, orient, spec{i});
    end
  end

  % Determine intrinsic width and height.
  w = sum(layout.prop.margin(1:2));
  if n > 0
    W = [layout.child.width];
    switch layout.prop.orient
     case 'horizontal'
      w = w + sum(W) + layout.prop.pad * (n - 1);
     case 'vertical'
      w = w + max(W);
    end
  end
  if layout.prop.width > w
    w = layout.prop.width;
  end

  h = sum(layout.prop.margin(3:4));
  if n > 0
    H = [layout.child.height];
    switch layout.prop.orient
     case 'horizontal'
      switch layout.prop.align
       case 'baseline'
        B = [layout.child.base];
        h = h + max(H + max(B) - B);
       otherwise
        h = h + max(H);
      end
     case 'vertical'
      h = h + sum(H) + layout.prop.pad * (n - 1);
    end
  end
  if layout.prop.height > h
    h = layout.prop.height;
  end

  layout.width = w;
  layout.height = h;

% mtk/grid
%
% [TODO]

% Render a grid.
function layout = mtki_render_grid(f, spec)

  error('Grid layout is not yet implemented');

% mtk/group
%
% A MTk group is a meta-widget of the following form:
%
%      {'group'
%       PROP, VALUE, ...
%       TOPLEVEL}
%
% Beside the 'Flexibility', 'Width', and 'Height' property, a group
% element has the following properties:
%
% Caption
%      The group label (a string).
%
% A group element only has one child element, a top-level layout element
% which describes the contents of the group.

% Render a group.
function layout = mtki_render_group(f, spec)

  % Create layout structure.
  layout = mtki_layout_struct('group');

  % Parse property/value pairs.
  layout.prop = struct('caption', [], ...
                       'width', 0, ...
                       'height', 0);

  % Ignore first element.
  ind = [1];

  n = numel(spec);
  for i = 2:2:n
    arg = spec{i};
    if iscell(arg)
      break;
    end
    if i == n
      error('Incomplete property/value pair');
    end
    capture_arg = 0;
    val = spec{i + 1};
    switch lower(arg)
     case 'caption'
      capture_arg = 1;
      if ischar(val)
        layout.prop.caption = val;
      else
        error('Invalid group caption');
      end
     case 'flexibility'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & val >= 0
        layout.flex = val;
      else
        error('Invalid group flexibility');
      end
     case 'width'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.width = val;
      else
        error('Invalid group width');
      end
     case 'height'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.height = val;
      else
        error('Invalid group height');
      end
    end
    % Save argument indices.
    if capture_arg
      ind = [ind, i, i + 1];
    end
  end

  spec(ind) = [];

  % Create group frame.
  layout.handle = mtk_frame(f, [0, 0, 43, 0, 43, 0]);
  if layout.prop.caption
    layout.handle(2) = mtk_text(f, [0, 0, 43, 0, 43, 0], ...
                                'String', layout.prop.caption);
  end

  n = numel(spec);
  switch n
   case 0
   case 1
    layout.child = mtki_render_top_level(f, 'vertical', spec{1});
   otherwise
    error('Wrong number of group children');
  end

  layout.width = layout.child.width;
  if layout.prop.width > layout.width
    layout.width = layout.prop.width;
  end

  layout.height = layout.child.height;
  if layout.prop.height > layout.height
    layout.height = layout.prop.height;
  end

  if layout.prop.caption
    pos = get(layout.handle(2), 'Position');
    % Add height of text widget.
    layout.height = layout.height + pos(4);
    % Set base line offset.
    layout.base = fix(pos(4) ./ 2);
  end

% mtk/tabs
%
% The MTk tab panel is a meta-widget of the following form:
%
%      {'tabs'
%       PROP, VALUE, ...
%       {'tab',
%        PROP, VALUE, ...
%        TOPLEVEL},
%       {'tab',
%        PROP, VALUE, ...
%        TOPLEVEL}}
%
% Beside the 'Flexibility', 'Width', and 'Height' property, a tabs
% element has the following properties:
%
% Value
%      The one-based index of the initially selected tab.  Default is
%      to display the contents of the first tab.
%
% MarkerColor
%      The color of the mark.  The mark is used to indicate the currently
%      selected tab.  Default is orange.
%
% The tabs element has one or more tab elements as children.  Each tab
% element only has one child element, a top-level layout element which
% describes the contents of the tab panel.  A tab element has the
% following properties:
%
% Caption
%      The tab label (a string).

% Render a tab panel.
function layout = mtki_render_tabs(f, spec)

  % Create layout structure.
  layout = mtki_layout_struct('tabs');

  % Parse property/value pairs.
  layout.prop = struct('value', 1, ...
                       'color', [1, 0.5, 0], ...
                       'width', 0, ...
                       'height', 0, ...
                       'tab', struct('caption', []));

  % Ignore first element.
  ind = [1];

  n = numel(spec);
  for i = 2:2:n
    arg = spec{i};
    if iscell(arg)
      break;
    end
    if i == n
      error('Incomplete property/value pair');
    end
    capture_arg = 0;
    val = spec{i + 1};
    switch lower(arg)
     case 'value'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & val >= 1
        layout.prop.value = fix(val);
      else
        error('Invalid tab value');
      end
     case 'markercolor'
      capture_arg = 1;
      if isempty(val) | ischar(val) | (isnumeric(val) & isreal(val) & numel(val) == 3 & all(val >= 0 & val <= 1))
        layout.prop.color = val;
      else
        error('Invalid tab marker color');
      end
     case 'flexibility'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & val >= 0
        layout.flex = val;
      else
        error('Invalid tab panel flexibility');
      end
     case 'width'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.width = val;
      else
        error('Invalid tab panel width');
      end
     case 'height'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.prop.height = val;
      else
        error('Invalid tab panel height');
      end
    end
    % Save argument indices.
    if capture_arg
      ind = [ind, i, i + 1];
    end
  end

  spec(ind) = [];

  n = numel(spec);
  if n < 1
    error('Too few tab elements');
  end

  if layout.prop.value < 1 | layout.prop.value > n
    error('Invalid tab value');
  end

  for j = 1:numel(spec)
    tab = spec{j};
    switch tab{1}
     case 'tab'
     otherwise
      error('Expect a ''tab'' element');
    end

    % Ignore first element.
    ind = [1];

    n = numel(tab);
    for i = 2:2:n
      arg = tab{i};
      if iscell(arg)
        break;
      end
      if i == n
        error('Incomplete property/value pair');
      end
      capture_arg = 0;
      val = tab{i + 1};
      switch lower(arg)
       case 'caption'
        capture_arg = 1;
        if ischar(val)
          layout.prop.tab(j).caption = val;
        else
          error('Invalid tab caption');
        end
      end
      % Save argument indices.
      if capture_arg
        ind = [ind, i, i + 1];
      end
    end

    tab(ind) = [];

    % Replace tab with panel contents.
    switch numel(tab)
     case 0
      spec{j} = {'box'};
     case 1
      spec{j} = tab{1};
     otherwise
      error('Invalid tab element');
    end
  end

  % Create tabs.
  n = numel(spec);

  layout.handle.tab = zeros(1, n);
  for i = 1:n
    layout.handle.tab(i) = mtk_pushbutton(f, [0, 0, 43, 0, 43, 0], ...
                                          'String', layout.prop.tab(i).caption);
  end

  % Create panel.
  layout.handle.panel = mtk_pushbutton(f, [1, 1, 43, 0, 43, 0], ...
                                       'Enable', 'inactive');

  % Create point and mark.
  layout.handle.point = mtk_text(f, [1, 1, 43, 0, 43, 0], ...
                                 'UserData', layout.handle.tab(layout.prop.value));

  if isempty(layout.prop.color)
    layout.handle.mark = [];
  else
    layout.handle.mark = mtk_text(f, [1, 1, 43, 0, 43, 0], ...
                                  'BackgroundColor', layout.prop.color);
  end

  % Render panel contents.
  layout.child = repmat(mtki_layout_struct([]), 1, n);
  for i = 1:n
    layout.child(i) = mtki_render_top_level(f, 'vertical', spec{i});

    % Remember widget visibility for each tab.
    tem = struct('handle', [], 'visible', []);
    tem.handle = mtki_layout_handles(layout.child(i));
    tem.visible = get(tem.handle, {'Visible'});

    % Hide panel contents of non-selected tabs.
    if i ~= layout.prop.value
      set(tem.handle, 'Visible', 'off');
    end

    % Install call-back procedure.
    set(layout.handle.tab(i), ...
        'Callback', {@mtki_render_tabs_post, layout.handle.point, layout.handle.mark}, ...
        'UserData', tem);
  end

  % Determine intrinsic width and height.
  tab_pos = zeros(n, 4);
  for i = 1:n
    tab_pos(i, :) = get(layout.handle.tab(i), 'Position');
  end

  layout.width = max(4 + sum(tab_pos(:, 3)), max([layout.child.width]));
  if layout.prop.width > layout.width
    layout.width = layout.prop.width;
  end

  layout.height = max([layout.child.height]) + tab_pos(1, 4) - 3;
  if layout.prop.height > layout.height
    layout.height = layout.prop.height;
  end

% Toggle display of tab panel contents.
function mtki_render_tabs_post(tab, event, point, mark)

  cur = get(point, 'UserData');
  if cur ~= tab
    % Move point and mark.
    mtki_layout_tab1(tab, point, mark);

    % Hide current panel contents.
    tem = get(cur, 'UserData');
    tem.visible = get(tem.handle, {'Visible'});
    set(tem.handle, 'Visible', 'off');
    set(cur, 'UserData', tem);

    % Show new panel contents.
    tem = get(tab, 'UserData');
    set(tem.handle, {'Visible'}, tem.visible);
  end

  mtk_focus(point);

% Render a widget.
function layout = mtki_render_widget(f, spec)

  % First element is the widget command.
  elem = spec{1};

  % Create layout structure.
  layout = mtki_layout_struct(elem);

  % Parse property/value pairs.
  % Ignore widget command.
  ind = [1];

  n = numel(spec);
  if rem(n, 2) ~= 1
    error('Wrong number of arguments');
  end

  for i = 2:2:n
    capture_arg = 0;
    val = spec{i + 1};
    switch lower(spec{i})
     case 'flexibility'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & val >= 0
        layout.flex = val;
      else
        error('Invalid widget flexibility');
      end
     case 'width'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.width = val;
      elseif ischar(val) | iscellstr(val)
        [layout.width, tem] = mtki_extent(val);
      else
        error('Invalid widget width');
      end
     case 'height'
      capture_arg = 1;
      if isnumeric(val) & isreal(val) & numel(val) == 1 & rem(val, 1) == 0 & val >= 0
        layout.height = val;
      elseif ischar(val) | iscellstr(val)
        [tem, layout.height] = mtki_extent(val);
      else
        error('Invalid widget height');
      end
    end
    % Save argument indices.
    if capture_arg
      ind = [ind, i, i + 1];
    end
  end

  spec(ind) = [];

  % Create widget.
  layout.handle = feval(['mtk_' elem], f, [layout.width, layout.height, 43, 0, 43, 0], spec{:});

  % Determine intrinsic width and height.
  pos = get(layout.handle, 'Position');

  layout.width = pos(3);
  layout.height = pos(4);

  % Set base line offset.
  switch elem
   case 'axes'
   case 'frame'
   case 'pushbutton'
    layout.base = 15;
   case 'togglebutton'
    layout.base = 15;
   case 'radiobutton'
    layout.base = 15;
   case 'checkbox'
    layout.base = 15;
   case 'popupmenu'
    layout.base = 15;
   case 'edit'
    layout.base = 15;
   case 'slider'
    layout.base = 15;
   case 'text'
    layout.base = 11;
   case 'listbox'
    layout.base = 15;
  end

% Place widgets.
function mtki_layout(f, layout, x, y, w, h)

  switch layout.elem
   case 'box'
    mtki_layout_box(f, layout, x, y, w, h);
   case 'grid'
    mtki_layout_grid(f, layout, x, y, w, h);
   case 'group'
    mtki_layout_group(f, layout, x, y, w, h);
   case 'tabs'
    mtki_layout_tabs(f, layout, x, y, w, h);
   otherwise
    mtk_move(layout.handle, f, ...
             [w, h, 43, x, 43, y]);
  end

% Box layout.
function mtki_layout_box(f, layout, x, y, w, h)

  % Place children.
  n = numel(layout.child);
  if n == 0
    return;
  end

  x = x + layout.prop.margin(1);
  y = y + layout.prop.margin(3);
  w = w - sum(layout.prop.margin(1:2));
  h = h - sum(layout.prop.margin(3:4));

  % Order of children.
  switch layout.prop.dir
   case 'normal'
    C = 1:n;
   case 'reverse'
    C = n:-1:1;
  end

  % Width and height of the children.
  W = [layout.child(C).width];
  H = [layout.child(C).height];

  % Padding between children.
  P = repmat(layout.prop.pad, 1, n - 1);

  switch layout.prop.orient
   case 'horizontal'
    % Distribute free space.
    d = w - sum(W) - sum(P);
    if d > 0
      F = [layout.child(C).flex];
      if any(F)
        W = W + fix(F ./ sum(F) .* d);
        % Remaining free space.
        d = w - sum(W) - sum(P);
      end
      if d > 0
        switch layout.prop.pack
         case 'start'
         case 'center'
          x = x + fix(d ./ 2);
         case 'end'
          x = x + d;
         case 'justify'
          if n > 1
            P = P + fix(d ./ (n - 1));
          end
        end
      end
    end

    % Left edge of children.
    X = [x, P; W];
    X = cumsum(X(:))';
    X = X(1:2:end);

    % Top edge of children.
    Y = repmat(y, 1, n);

    if strcmp(layout.prop.align, 'baseline')
      B = [layout.child(C).base];
      Y = Y + max(B) - B;
    else
      d = h - H;
      if any(d)
        switch layout.prop.align
         case 'start'
         case 'center'
          Y = Y + fix(d ./ 2);
         case 'end'
          Y = Y + d;
         case 'stretch'
          H = repmat(h, 1, n);
        end
      end
    end

   case 'vertical'
    % Distribute free space.
    d = h - sum(H) - sum(P);
    if d > 0
      F = [layout.child(C).flex];
      if any(F)
        H = H + fix(F ./ sum(F) .* d);
        % Remaining free space.
        d = h - sum(H) - sum(P);
      end
      if d > 0
        switch layout.prop.pack
         case 'start'
         case 'center'
          y = y + fix(d ./ 2);
         case 'end'
          y = y + d;
         case 'justify'
          if n > 1
            P = P + fix(d ./ (n - 1));
          end
        end
      end
    end

    % Top edge of children.
    Y = [y, P; H];
    Y = cumsum(Y(:))';
    Y = Y(1:2:end);

    % Left edge of children.
    X = repmat(x, 1, n);

    d = w - W;
    if any(d)
      switch layout.prop.align
       case 'start'
       case 'center'
        X = X + fix(d ./ 2);
       case 'end'
        X = X + d;
       case 'stretch'
        W = repmat(w, 1, n);
      end
    end
  end

  for i = 1:n
    mtki_layout(f, layout.child(C(i)), X(i), Y(i), W(i), H(i));
  end

% Grid layout.
function mtki_layout_grid(f, layout, x, y, w, h)

  error('Grid layout is not yet implemented');

% Group layout.
function mtki_layout_group(f, layout, x, y, w, h)

  if layout.prop.caption
    % Center text widget vertically in the frame border.
    pos = get(layout.handle(2), 'Position');

    mtk_move(layout.handle(1), f, ...
             [w, h - layout.base, 43, x, 43, y + layout.base]);

    mtk_move(layout.handle(2), layout.handle(1), ...
             [0, 0, 43, 4, 43, 0 - layout.base]);

    y = y + pos(4);
    h = h - pos(4);
  else
    mtk_move(layout.handle, f, ...
             [w, h, 43, x, 43, y]);
  end

  if numel(layout.child) > 0
    mtki_layout(f, layout.child, x, y, w, h);
  end

% Tab panel layout.
function mtki_layout_tabs(f, layout, x, y, w, h)

  % Number of tabs.
  n = numel(layout.child);

  % Place tabs.
  mtk_move(layout.handle.tab(1), f, ...
           [0, 0, 43, x + 2, 43, y + (1 - ispc)]);

  for i = 2:n
    mtk_move(layout.handle.tab(i), layout.handle.tab(i - 1), ...
             [0, 0, 62, 0, 43, 0]);
  end

  % Place point and mark.
  mtki_layout_tab1(layout.handle.tab(layout.prop.value), ...
                   layout.handle.point, layout.handle.mark);

  % Place tab panel.
  y = y + 20;
  h = h - 20;

  mtk_move(layout.handle.panel, f, ...
           [w, h, 43, x, 43, y]);

  % Place children.
  for i = 1:n
    mtki_layout(f, layout.child(i), x, y, w, h);
  end

function mtki_layout_tab1(tab, point, mark)

  % Get tab position.
  tem = get(tab, 'Position');

  % Move point.
  if ispc
    pos(1) = tem(1) + 1;
    pos(2) = tem(2);
    pos(3) = tem(3) - 2;
    pos(4) = 2;
  else
    pos(1) = tem(1) + 3;
    pos(2) = tem(2);
    pos(3) = tem(3) - 6;
    pos(4) = 3;
  end

  set(point, 'Position', pos, 'UserData', tab);

  % Move mark.
  if mark
    pos(1) = tem(1) + 1;
    pos(2) = tem(2) + 2;
    pos(3) = tem(3) - 2;
    pos(4) = 1;

    set(mark, 'Position', pos);
  end

% Return a layout structure.
function layout = mtki_layout_struct(elem)

  layout = struct('elem', elem, ...
                  'prop', [], ...
                  'child', [], ...
                  'handle', [], ...
                  'flex', 0.0, ...
                  'width', 0, ...
                  'height', 0, ...
                  'base', 0);

% Return all widget handles.
function h = mtki_layout_handles(layout)

  h = [];

  if isstruct(layout.handle)
    f = fieldnames(layout.handle);
    for i = 1:numel(f)
      v = getfield(layout.handle, f{i});
      h = [h; v(:)];
    end
  else
    h = layout.handle(:);
  end

  for i = 1:numel(layout.child)
    h = [h; mtki_layout_handles(layout.child(i))];
  end

% Resize function.
function mtki_resize(obj, event)

  layout = getappdata(obj, 'mtk_layout');
  if isempty(layout)
    return;
  end

  % Get requested figure size.
  pos = get(obj, 'Position');

  % Don't get smaller than the intrinsic size.
  if pos(3) < layout.width
    pos(3) = layout.width;
  end
  if pos(4) < layout.height
    pos(4) = layout.height;
  end

  % Don't resize the width or height if the figure's
  % top-level box is not flexible.
  switch layout.elem
   case 'box'
    switch layout.prop.orient
     case 'horizontal'
      if layout.flex == 0
        pos(3) = layout.width;
      end
      if numel(layout.child) > 0 & all([layout.child.flex] == 0)
        pos(4) = layout.height;
      end
     case 'vertical'
      if numel(layout.child) > 0 & all([layout.child.flex] == 0)
        pos(3) = layout.width;
      end
      if layout.flex == 0
        pos(4) = layout.height;
      end
    end
  end

  % Set actual figure size.
  set(obj, 'Position', pos);

  % Apply widget layout on the new size.
  mtki_layout(obj, layout, 0, 0, pos(3), pos(4));

% mtk/figure
%
%      h = mtk('figure', master, geometry, ...)
%
% Create a figure widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the figure widget.
function h = mtk_figure(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('figure', ...
                  master, geometry, varargin);

  % Smart figure placement.
  movegui(h, 'onscreen');

% mtk/axes
%
%      h = mtk('axes', master, geometry, ...)
%
% Create an axes widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the axes widget.
function h = mtk_axes(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('axes', ...
                  master, geometry, varargin);

% mtk/pushbutton
%
%      h = mtk('pushbutton', master, geometry, ...)
%
% Create a push button widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted as
% property/value pairs for the widget.
%
% Return value is the handle of the push button widget.
function h = mtk_pushbutton(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('pushbutton', ...
                  master, geometry, varargin);

% mtk/togglebutton
%
%      h = mtk('togglebutton', master, geometry, ...)
%
% Create a toggle button widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted as
% property/value pairs for the widget.
%
% Return value is the handle of the toggle button widget.
function h = mtk_togglebutton(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('togglebutton', ...
                  master, geometry, varargin);

% mtk/radiobutton
%
%      h = mtk('radiobutton', master, geometry, ...)
%
% Create a radio button widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted as
% property/value pairs for the widget.
%
% Return value is the handle of the radio button widget.
function h = mtk_radiobutton(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('radiobutton', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left');

% mtk/checkbox
%
%      h = mtk('checkbox', master, geometry, ...)
%
% Create a check box widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the check box widget.
function h = mtk_checkbox(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('checkbox', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left');

% mtk/edit
%
%      h = mtk('edit', master, geometry, ...)
%
% Create an edit widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the edit widget.
function h = mtk_edit(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('edit', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left', ...
                  'BackgroundColor', [1 1 1]);

% mtk/text
%
%      h = mtk('text', master, geometry, ...)
%
% Create a text widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the text widget.
function h = mtk_text(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('text', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left');

% mtk/slider
%
%      h = mtk('slider', master, geometry, ...)
%
% Create a slider widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the slider widget.
function h = mtk_slider(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('slider', ...
                  master, geometry, varargin);

% mtk/frame
%
%      h = mtk('frame', master, geometry, ...)
%
% Create a frame widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the frame widget.
function h = mtk_frame(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('frame', ...
                  master, geometry, varargin);

% mtk/listbox
%
%      h = mtk('listbox', master, geometry, ...)
%
% Create a list box widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted
% as property/value pairs for the widget.
%
% Return value is the handle of the list box widget.
function h = mtk_listbox(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('listbox', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left', ...
                  'BackgroundColor', [1 1 1]);

% mtk/popupmenu
%
%      h = mtk('popupmenu', master, geometry, ...)
%
% Create a pop-up menu widget and place it relative to MASTER (a widget
% handle) at position GEOMETRY.  Additional arguments are interpreted as
% property/value pairs for the widget.
%
% Return value is the handle of the pop-up menu widget.
function h = mtk_popupmenu(master, geometry, varargin)

  % Check number of arguments.
  error(nargchk(2, inf, nargin));

  % Create Handle Graphics object.
  h = mtki_widget('popupmenu', ...
                  master, geometry, varargin, ...
                  'HorizontalAlignment', 'left');

% mtk/menu
%
%      h = mtk('menu', master, ...)
%
% Create a menu widget in the figure or menu MASTER.  Additional arguments
% are interpreted as property/value pairs for the widget.
%
% If the value of the 'Label' property contains a '^' character, then the
% character following the '^' character specifies the accelerator key for
% the menu item (this is consistent with the 'makemenu' command).
%
% Return value is the handle of the menu widget.
function h = mtk_menu(master, varargin)

  % Check number of arguments.
  error(nargchk(1, inf, nargin));

  % Create Handle Graphics object.
  h = uimenu('Parent', master);

  % Apply extra properties.
  n = numel(varargin);
  if n > 0
    key = [];
    for i = 1:2:n
      switch lower(varargin{i})
       case 'label'
        [varargin{i+1}, key] = mtki_label(varargin{i+1});
      end
    end
    if ~ isempty(key)
      set(h, 'Accelerator', key);
    end
    set(h, varargin{:});
  end

% mtk/menuitem
%
%      h = mtk('menuitem', master, ...)
%
% Create menu items in the figure, menu, or context menu MASTER.
% Additional arguments are sub-menus or menu items.  A sub-menu is a
% cell array whose first element is the menu item and the rest of the
% elements are sub menus and menu items.  A menu item is either a menu
% label, a cell array whose first element is the menu label and the
% second element is the call-back function handle (further cell array
% elements are additional arguments for the call-back function), or a
% character array whose first row is the menu label and the second row
% is the call-back string.  A menu label is a string.  See the MTk menu
% command, for more details about menu labels.
%
% Return value is a vector with the handles of the top-level menu widgets.
%
% Example:
%
%      f = mtk('figure', 0, [300 200], ...
%              'Name', 'MTk Demo');
%
%      mtk('menuitem', f, ...
%          {'&File', ...
%           strvcat('&Save^s', 'filemenufcn(gcbf, ''FileSave'')'), ...
%           strvcat('Save as...', 'filemenufcn(gcbf, ''FileSaveAs'')'), ...
%           strvcat('Export...', 'filemenufcn(gcbf, ''FileExport'')'), ...
%           '--', ...
%           strvcat('Page setup...', 'pagesetupdlg(gcbf)'), ...
%           strvcat('Print setup...', 'printdlg(''-setup'', gcbf)'), ...
%           strvcat('Print preview...', 'printpreview(gcbf)'), ...
%           strvcat('&Print...', 'printdlg(gcbf)'), ...
%           '--' ...
%           {'Exit', @exit_callback}}, ...
%          {'&Help', ...
%           strvcat('Demos', 'demo')});
function h = mtk_menuitem(master, varargin)

  % Check number of arguments.
  error(nargchk(1, inf, nargin));

  n = numel(varargin);
  if n == 0
    h = [];
  else
    % Maximum number of menu items.
    h = repmat(nan, 1, n);
    % Turn off separator flag.
    sep = 'off';
    % Create menu items.
    for i = 1:n
      [h(i), sep] = mtki_menuitem1(master, sep, varargin{i});
    end
    % Delete superfluous handles.
    ind = find(isnan(h));
    if ind
      h(ind) = [];
    end
  end

% Create menu items (stage 1).
function [h, sep] = mtki_menuitem1(h1, sep1, this)

  switch class(this)
   case 'cell'
    n = numel(this);
    switch n
     case 0
      error('Empty menu items are not allowed');
     case 1
      error('Superfluous nesting of menu items');
     otherwise
      switch class(this{2})
       case 'function_handle'
        [h, sep] = mtki_menuitem2(h1, sep1, this{:});
       otherwise
        [h, sep] = mtki_menuitem2(h1, sep1, this{1});
        for i = 2:n
          [tem, sep] = mtki_menuitem1(h, sep, this{i});
        end
        sep = 'off';
      end
    end
   case 'char'
    if isempty(this) | all(this(:) == '-')
      [h, sep] = deal(nan, 'on');
    else
      [h, sep] = mtki_menuitem2(h1, sep1, this);
    end
   otherwise
    error('Invalid menu item');
  end

% Create menu items (stage 2).
function [h, sep] = mtki_menuitem2(h1, sep1, label, varargin)

  if ~ ischar(label)
    error('Invalid menu label');
  end

  if isempty(label) | all(label(:) == '-')
    error('Unexpected menu item separator');
  end

  if numel(varargin) > 0
    callback = varargin;
  else
    switch size(label, 1)
     case 1
      callback = [];
     case 2
      tem = cellstr(label);
      label = tem{1};
      callback = tem{2};
     otherwise
      error('Invalid menu item');
    end
  end

  [label, key] = mtki_label(label);

  % Create Handle Graphics object.
  h = uimenu('Parent', h1, ...
             'Separator', sep1, ...
             'Label', label);

  if ~ isempty(key)
    set(h, 'Accelerator', key);
  end
  if ~ isempty(callback)
    set(h, 'Callback', callback);
  end

  sep = 'off';

% Parse a menu label.
function [label, key] = mtki_label(string)

  % Convert old-style keyboard equivalent escape sequences.
  ind = strfind(string, '\&');
  if ind
    string(ind) = '&';
  end

  % Extract accelerator key.  Use last key specified and make
  % it unique on all platforms.
  ind = find(string == '^');
  if isempty(ind)
    key = [];
  else
    key = string(ind+1);
    if isunix
      key = lower(key(end));
    else
      key = upper(key(end));
    end
    % Delete '^'.
    string(ind) = [];
    % Delete key.
    string(ind) = [];
  end

  label = string;

% mtk/contextmenu
%
%      h = mtk('contextmenu', master, ...)
%
% Create a context menu widget for the graphics object MASTER.  Additional
% arguments are interpreted as property/value pairs for the widget.
%
% Return value is the handle of the context menu widget.
function h = mtk_contextmenu(master, varargin)

  % Check number of arguments.
  error(nargchk(1, inf, nargin));

  % Determine parent of master widget.
  switch get(master, 'Type')
   case 'figure'
    parent = master;
   case {'axes', 'uicontrol', 'uimenu'}
    parent = get(master, 'Parent');
  end

  % Create Handle Graphics object.
  h = uicontextmenu('Parent', parent);

  % Apply extra properties.
  if numel(varargin) > 0
    set(h, varargin{:});
  end

  % Associate context menu with master widget.
  set(master, 'UIContextMenu', h);

% Create a MTk widget.
function h = mtki_widget(style, master, geometry, properties, varargin)

  persistent background

  % Initialize constants.
  if isempty(background)
    f = figure('Visible', 'off');
    t = uicontrol('Parent', f, ...
                  'Style', 'frame');
    % Default background color.
    background = get(t, 'BackgroundColor');
    delete(f);
  end

  % Parse widget geometry.
  geometry = mtki_geometry(geometry);

  if geometry(1) == 0
    switch style
     case 'figure'
      geometry(1) = 320;
     case {'axes', ...
           'frame'}
      pos = get(master, 'Position');
      geometry(1) = pos(3);
     case {'pushbutton', ...
           'togglebutton'}
      for i = 1:2:numel(properties)
        switch lower(properties{i})
         case 'string'
          geometry(1) = mtki_extent(properties{i+1}) + 16;
        end
      end
     case {'radiobutton', ...
           'checkbox', ...
           'popupmenu'}
      for i = 1:2:numel(properties)
        switch lower(properties{i})
         case 'string'
          geometry(1) = mtki_extent(properties{i+1}) + 24;
        end
      end
     case {'edit', ...
           'slider'}
      geometry(1) = 80;
     case 'text'
      for i = 1:2:numel(properties)
        switch lower(properties{i})
         case 'string'
          geometry(1) = mtki_extent(properties{i+1});
        end
      end
     case 'listbox'
      for i = 1:2:numel(properties)
        switch lower(properties{i})
         case 'string'
          geometry(1) = mtki_extent(properties{i+1}) + 24;
        end
      end
    end
  end
  if geometry(2) == 0
    switch style
     case 'figure'
      geometry(2) = 240;
     case {'axes', ...
           'frame'}
      pos = get(master, 'Position');
      geometry(2) = pos(4);
     case {'pushbutton', ...
           'togglebutton', ...
           'radiobutton', ...
           'checkbox', ...
           'popupmenu', ...
           'edit', ...
           'slider'}
      geometry(2) = 22;
     case 'text'
      geometry(2) = 14;
     case 'listbox'
      geometry(2) = 120;
    end
  end

  % Convert widget dimensions.
  [parent, position] = mtki_transform(style, master, geometry);

  % Create widget.
  switch style
   case 'figure'
    h = figure('CloseRequestFcn', 'closereq;', ...
               'HandleVisibility', 'callback', ...
               'IntegerHandle', 'off', ...
               'NumberTitle', 'off', ...
               'MenuBar', 'none', ...
               'Color', background, ...
               'Resize', 'off', ...
               'Units', 'pixels', ...
               'Position', position);
   case 'axes'
    h = axes('Parent', parent, ...
             'Units', 'pixels', ...
             'Position', position);
   otherwise
    h = uicontrol('Parent', parent, ...
                  'Style', style, ...
                  'Units', 'pixels', ...
                  'Position', position);
  end

  % Apply extra properties.
  if numel(varargin) > 0
    set(h, varargin{:});
  end
  if numel(properties) > 0
    set(h, properties{:});
  end

% mtk/move
%
%      mtk('move', widget, master, geometry)
%
% Move the graphics object WIDGET relative to MASTER (a widget handle)
% at position GEOMETRY.
%
% The move command resets the widget units to pixels.
function h = mtk_move(widget, master, geometry)

  % Check number of arguments.
  error(nargchk(3, 3, nargin));

  % Widget style.
  style = mtki_style(widget);

  % Current widget dimensions.
  position = mtki_position(widget);

  % Parse widget geometry.
  geometry = mtki_geometry(geometry);

  if geometry(1) == 0
    geometry(1) = position(3);
  end
  if geometry(2) == 0
    geometry(2) = position(4);
  end

  % Convert widget dimensions.
  [parent, position] = mtki_transform(style, master, geometry);

  % Move widget.
  set(widget, 'Parent', parent, ...
              'Units', 'pixels', ...
              'Position', position);

% mtk/geometry
%
% The origin of MTk's coordinate system is at the upper left corner of
% the screen at position (0, 0).  The x-axis is horizontal and is numbered
% from left to right, and the y-axis is vertical and is numbered from top
% to bottom.
%
% The internal representation of a widget geometry is a vector with six
% elements.
%
%      [WIDTH HEIGHT HALIGN HOFFSET VALIGN VOFFSET]
%
% First element WIDTH is the width of the widget.
% Second element HEIGHT is the height of the widget.
% Third element HALIGN specifies the horizontal alignment of the widget
%  to its master.  Possible values are '+', '-', '<', '=', and '>'.
% Fourth element HOFFSET is the distance between the horizontal alignment
%  positions.
% Fifth element VALIGN specifies the vertical alignment of the widget
%  to its master.  Possible values are '+', '-', '<', '=', and '>'.
% Sixth element VOFFSET is the distance between the vertical alignment
%  positions.
%
% The external representation of a widget geometry is a string of the
% form
%
%      'WIDTH x HEIGHT HALIGN HOFFSET VALIGN VOFFSET'
%
% WIDTH, HEIGHT, HOFFSET, and VOFFSET are numbers.
% HALIGN and VALIGN can be one of '+', '-', '<', '=', or '>'.
%  A value of '=' specifies center alignment, '+' aligns the left side
%  (top) of the widget to the left side (top) of the master, '<' aligns
%  the right side (bottom) of the widget to the left side (top) of the
%  master, '-' aligns the right side (bottom) of the widget to the right
%  side (bottom) of the master, and '>' aligns the left side (top) of the
%  widget to the right side (bottom) of the master.
%  When placing a widget inside a master, '+0+0' specifies the upper left
%  corner, '+0-0' the lower left corner, '-0+0' the upper right corner,
%  and '-0-0' the lower right corner.
%
% If only the width and height is specified, the widget is centered.
function geometry = mtki_geometry(spec)

  if ischar(spec)
    geometry = sscanf(spec, '%d x %d %1[<=>+-] %d %1[<=>+-] %d %c');
  else
    geometry = spec;
  end

  switch numel(geometry)
   case 2
    geometry(3:6) = ['=' 0 '=' 0];
   case 6
    switch geometry(3)
     case 43 % +
     case 45 % -
     case 60 % <
     case 61 % =
     case 62 % >
     otherwise
      error('Invalid horizontal alignment character');
    end
    switch geometry(5)
     case 43 % +
     case 45 % -
     case 60 % <
     case 61 % =
     case 62 % >
     otherwise
      error('Invalid vertical alignment character');
    end
   otherwise
    error('Invalid widget geometry');
  end

  if geometry(1) < 0
    error('Invalid widget width');
  end
  if geometry(2) < 0
    error('Invalid widget height');
  end

% Convert widget dimensions.
function [parent, position] = mtki_transform(style, master, geometry)

  % Widget style of the master.
  master_style = mtki_style(master);

  % Original position of the master within its parent.
  position = mtki_position(master) - mtki_offset(master_style);

  % Determine parent object.
  switch master_style
   case 'root'
    switch style
     case 'figure'
      parent = master;
     otherwise
      error('Only figures can be children of the root object');
    end
   case 'figure'
    switch style
     case 'figure'
      parent = 0;
     otherwise
      parent = master;
      % Descendants of figure objects are placed relative
      % to the figure's origin.
      position(1:2) = [1 1];
    end
   otherwise
    parent = get(master, 'Parent');
  end

  % Coordinate transformation.
  switch geometry(3)
   case 43 % +
    x = position(1) + geometry(4);
   case 45 % -
    x = position(1) - geometry(4) + position(3) - geometry(1);
   case 60 % <
    x = position(1) - geometry(4) - geometry(1);
   case 62 % >
    x = position(1) + geometry(4) + position(3);
   otherwise
    x = position(1) + geometry(4) + fix((position(3) - geometry(1)) / 2);
  end
  switch geometry(5)
   case 43 % +
    y = position(2) - geometry(6) + position(4) - geometry(2);
   case 45 % -
    y = position(2) + geometry(6);
   case 60 % <
    y = position(2) + geometry(6) + position(4);
   case 62 % >
    y = position(2) - geometry(6) - geometry(2);
   otherwise
    y = position(2) - geometry(6) + fix((position(4) - geometry(2) + 1) / 2);
  end
  position = [x y geometry(1) geometry(2)] + mtki_offset(style);

% Determine widget style for Handle Graphics object H.
function style = mtki_style(h)

  style = get(h, 'Type');
  switch style
   case 'root'
   case 'figure'
   case 'axes'
   case 'uicontrol'
    style = get(h, 'Style');
   otherwise
    error('Invalid argument');
  end

% Return position of the Handle Graphics object H.
function position = mtki_position(h)

  if h == 0
    units = get(0, 'Units');
    set(0, 'Units', 'pixels');
    position = get(0, 'ScreenSize');
    set(0, 'Units', units);
  else
    units = get(h, 'Units');
    set(h, 'Units', 'pixels');
    position = get(h, 'Position');
    set(h, 'Units', units);
  end

% Return Handle Graphics offset for a widget of type STYLE.
function offset = mtki_offset(style)

  % Default offset.
  offset = [0 0 0 0];

  switch style
   case 'frame'
    % Make frame borders overlap nicely.
    if isunix
      offset = [-1 -1 2 2];
    else
      switch computer
       case 'PCWIN'
        offset = [-1 0 1 1];
      end
    end
  end

% Return background color of the Handle Graphics object H.
function color = mtki_background(h)

  switch get(h, 'Type')
   case 'figure'
    color = get(h, 'Color');
   case 'axes'
    color = get(h, 'Color');
    if ischar(color) & strcmp(color, 'none')
      h = get(h, 'Parent');
      color = get(h, 'Color');
    end
   case 'uicontrol'
    color = get(h, 'BackgroundColor');
   otherwise
    error('Invalid Handle Graphics object');
  end

% Get MTk application data.
function varargout = mtki_get_data(h, varargin)

  data = getappdata(h, 'mtk_data');
  if isempty(data)
    data = struct('master', [], ...
                  'geometry', []);
  end

  if nargin == 1
    varargout = {data};
  else
    n = numel(varargin);
    varargout = cell(1, n);
    for i = 1:n
      varargout{i} = getfield(data, varargin{i});
    end
  end

% Set MTk application data.
function mtki_set_data(h, varargin)

  if nargin == 2
    data = varargin{1};
  else
    n = numel(varargin);
    data = mtki_get_data(h);
    for i = 1:2:n
      data = setfield(data, varargin{i}, varargin{i+1});
    end
  end

  setappdata(h, 'mtk_data', data);

%% mtk.m ends here
